// AddressBook.java
// Copyright 1996 Netscape Communications Corp.  All rights reserved.

/** This example application implements a simple address book.
  * A subclass of ListView called AddressView is created which uses a subclass
  * of ListItem called AddressItem.  Each AddressItem draws a set of text
  * strings containing names, addresses, phone numbers, etc.  If the user
  * double-clicks on a field, a TextField is temporarily superimposed over the
  * field in order to allow editing of the field.  The tab key can be used
  * to move from field to field within an entry.
  *
  * A separate TextField is used to do an incremental seach of the fields.
  * Key events are filtered by the AddressView, and used to carry out the
  * search as each key is typed.
  *
  * A TextView is used to display and edit a general info field associated
  * with each address entry.
  *
  * Finally, a set of Buttons (with a custom drawing method) is used to add
  * and delete entries, to allow loading from a file, and to allow saving
  * to a file when run stand-alone or from the appletviewer.
  *
  * This example primarily illustrates the use of the following classes
  * and concepts:
  *
  *     ListView and ListItem
  *     ScrollGroup and ScrollView
  *     TextField and TextView
  *     TextFilter
  *     Button
  *     Targets and Commands
  *
  * Secondarily, it also illustrates:
  *
  *     InternalWindow
  *     Alert 
  *     File saving and loading
  */

import netscape.application.*;
import netscape.util.*;
import java.awt.AWTError;
import java.io.*;
import java.net.*;

public class AddressBook extends Application implements Target {
    AddressView    addressView;           // Manages the list of addresses
    ScrollGroup    infoGroup;             // Manages scrolling of info text
    FileViewer     fileViewer;            // The file selection window

    // --------------  Application initialization  --------------

    public void init() {
        InternalWindow mainWindow;        // Our one and only window
        ScrollGroup    addressGroup;      // Manages scrolling of addresses
        TextField      searchField;       // Used for searching the addresses
        TextField      label;             // Used for creating text labels
        TextView       infoView;          // Displays the info text
        RoundedButton  deleteButton;      // Deletes address entries
        RoundedButton  newButton;         // Creates new address entries
        RoundedButton  saveButton;        // Writes addresses to a file
        RoundedButton  loadButton;        // Reads addresses from a file

        super.init();

        // Give ourselves a nice white marble background
        mainRootView().setImage(Bitmap.bitmapNamed("wmarble.gif"));
        mainRootView().setImageDisplayStyle(Image.TILED);

        // Create the main window and make it resizable
        mainWindow = new InternalWindow(20, 20, 500, 400);
        mainWindow.setResizable(true);
        mainWindow.setTitle("Address Book");

        // Give the window a dark blue-gray background color
        mainWindow.contentView().setColor(new Color(32, 32, 64));

        // Create the four buttons for New, Delete, Load, and Save
        // Because we set this object to be the target, it will
        // receive a performCommand() message whenever a button
        // is pushed (see below)
        newButton = new RoundedButton(10, 10, 95, 35);
        newButton.setTitle("New Entry");
        newButton.setTitleColor(Color.cyan);
        newButton.setTarget(this);
        newButton.setCommand("newItem");
        mainWindow.addSubview(newButton);

        deleteButton = new RoundedButton(115, 10, 95, 35);
        deleteButton.setTitle("Delete Entry");
        deleteButton.setTitleColor(Color.red);
        deleteButton.setTarget(this);
        deleteButton.setCommand("delete");
        mainWindow.addSubview(deleteButton);

        saveButton = new RoundedButton(10, 50, 95, 35);
        saveButton.setTitle("Save Entries");
        saveButton.setTitleColor(Color.magenta);
        saveButton.setTarget(this);
        saveButton.setCommand("save");
        mainWindow.addSubview(saveButton);

        loadButton = new RoundedButton(115, 50, 95, 35);
        loadButton.setTitle("Load Entries");
        loadButton.setTitleColor(Color.blue);
        loadButton.setTarget(this);
        loadButton.setCommand("load");
        mainWindow.addSubview(loadButton);

        // This is used for incremental searches
        searchField = new TextField(10, 95, 200, 30);
        mainWindow.addSubview(searchField);

        // Just a label to identify the searchField
        label = new TextField(10, 126, 200, 20);
        label.setTransparent(true);
        label.setEditable(false);
        label.setJustification(Graphics.CENTERED);
        label.setStringValue("Search Text");
        label.setTextColor(Color.white);
        mainWindow.addSubview(label);

        // A ScrollGroup manages a ScrollView and ScrollBars for its
        // specified content view.  This one handles scrolling the
        // info TextView
        infoGroup = new ScrollGroup(250, 10, 230, 115);
        infoGroup.setHasVertScrollBar(true); // Vertical only
        infoGroup.setBorder(new BezelBorder(BezelBorder.LOWERED,
                                            Color.lightGray));
        infoGroup.scrollView().setBackgroundColor(Color.white);
        // The following causes the bottom of the group to be fixed
        // (i.e., it's margin, the space below it, can stretch)
        infoGroup.setVertResizeInstruction(View.BOTTOM_MARGIN_CAN_CHANGE);
	infoGroup.setHorizResizeInstruction(View.WIDTH_CAN_CHANGE);
        infoView = new TextView(0, 0, 210, 200);
        infoView.setBackgroundColor(Color.white);
        infoGroup.scrollView().setContentView(infoView);
        mainWindow.addSubview(infoGroup);

        // Just a label to identify the info field
        label = new TextField(250, 126, 230, 20);
        label.setTransparent(true);
        label.setEditable(false);
        label.setJustification(Graphics.CENTERED);
        label.setStringValue("Info");
        label.setTextColor(Color.white);
        // This allows the label to stretch horizontally when the window is
        // resized (TextFields don't stretch by default)
        label.setHorizResizeInstruction(View.WIDTH_CAN_CHANGE);
        mainWindow.addSubview(label);

        // This ScrollGroup manages scrolling of the address list
        addressGroup = new ScrollGroup(10, 150, 470, 210);
        addressGroup.setHasHorizScrollBar(true);
        addressGroup.setHasVertScrollBar(true);
        addressGroup.setBorder(new BezelBorder(BezelBorder.LOWERED,
                                               Color.lightGray));
        // Using n off-screen drawing buffer makes scrolling appear smoother
        addressGroup.scrollView().setBuffered(true);
        addressGroup.scrollView().setBackgroundColor(
                                  new Color(235, 225, 210));

        addressView = new AddressView(0, 0, 750, 300);
        addressView.setBackgroundColor(new Color(235, 225, 210));
        addressGroup.scrollView().setContentView(addressView);
	addressGroup.setHorizResizeInstruction(View.WIDTH_CAN_CHANGE);
	addressGroup.setVertResizeInstruction(View.HEIGHT_CAN_CHANGE);
        mainWindow.addSubview(addressGroup);

        // The AddressView needs to know where to display the info when
        // an address entry is selected
        addressView.setInfoEditor(infoView);

        // The AddressView also checks each character that is typed into the
        // searchField, matching the contents to address fields
        searchField.setFilter(addressView);

        // Ok, let's see it!
        mainWindow.show();
   }

    // Create a new item
    public void newItem(Object anObject) {
	addressView.newItem();	
    }

    // Delete the selectedItem
    public void delete(Object anObject) {
	addressView.delete();
	infoGroup.scrollView().draw();
    }

    // Save with the FileChooser 
    public void save(Object anObject) {
	selectFile(false);
    }

    // Load with the FileChooser
    public void load(Object anObject) {
	selectFile(true);	
    }

    // Load the selected file
    public void loadFileSelected(Object anObject) {
	loadFile(fileViewer.fileURL());
    }

    // Save to the selected file
    public void saveFileSelected(Object anObject) {
	saveFile(fileViewer.fileURL());	
    }
    // ----------------- Target interface method  -----------------

    // Here we dispatch messages sent by the buttons when they
    // are pushed.  The command string is the string set by
    // the setCommand() calls in the init() method above.
    public void performCommand(String command, Object anObject) {
        if (command.equals("newItem")) {
	    newItem(anObject);
        }
        else if (command.equals("delete")) {
	    delete(anObject);
        }
        else if (command.equals("save")) {
	    save(anObject);
        }
        else if (command.equals("load")) {
	    load(anObject);
        }
        else if (command.equals("loadFileSelected")) {
	    loadFileSelected(anObject);
        }
        else if (command.equals("saveFileSelected")) {
	    saveFileSelected(anObject);
        }
    }


    // ------------  File loading and saving methods  ------------

    // Bring up the file viewer set up to show .add files
    // This will by default show files in the applet's base directory
    // The viewer will send a command back to us when the user selects a file
    void selectFile(boolean load) {
        if (fileViewer == null) {
            fileViewer = new FileViewer(50, 50, 300, 300);
            fileViewer.setTarget(this);
            fileViewer.setRequiredExtension("add");
        }
        else if (fileViewer.isVisible())
            return;
        fileViewer.setTitle("AddressBook - " + (load ? "Load" : "Save"));
        fileViewer.setCommand(load ? "loadFileSelected" : "saveFileSelected");
        fileViewer.showModally();
    }

    // This is called when a user selects a file to load.
    // Open a stream to the specified URL and pass it to the AddressView.
    void loadFile(URL url) {
        if (url == null)
            return;
        try {
            InputStream stream;

            stream = url.openStream();
            addressView.load(stream);
            stream.close();
        }
        catch (IOException e) {
	    Alert.runAlertInternally(Alert.warningImage(),"Open",
				     "I/O Error while opening " + url.getFile() +
				     ": " + e,"OK",null,null);
        }
    }

    // This is called when a user selects a file to save.
    // Open a stream to the specified URL and pass it to the AddressView.
    void saveFile(URL url) {
        OutputStream stream = null;

        if (url == null)
            return;

        // We can only write to the local file system
        if (!url.getProtocol().equals("file"))
            return;

        // Open a stream to the url
        try {
            stream = new FileOutputStream(url.getFile());
        }
        catch (IOException e) {
	    Alert.runAlertInternally(Alert.warningImage(),"Save",
				     "I/O Error while saving " + url.getFile() +
				     ": " + e,"OK",null,null);
            return;
        }
        catch (Exception e) {
	    Alert.runAlertInternally(Alert.warningImage(),"Save",
				     "Error while saving " + url.getFile() +
				     ": " + e,"OK",null,null);
            return;
        }

        // Save the current entries
        if (stream != null) {
            addressView.save(stream);
            try {
                stream.close();
            }
            catch (IOException e) {
		Alert.runAlertInternally(Alert.warningImage(),"Save",
				     "I/O Error while saving " + url.getFile() +
				     ": " + e,"OK",null,null);
            }
            catch (Exception e) {
		Alert.runAlertInternally(Alert.warningImage(),"Save",
				     "Error while saving " + url.getFile() +
				     ": " + e,"OK",null,null);
                return;
            }
        }
    }


    //  -------------  Stand-alone application methods  ------------------

    // Called by Java when we're run as a stand-alone application
    public static void main(String args[]) {
        AddressBook    app;
        Size           size;
        ExternalWindow mainWindow;

        // Create an instance and run it in an external window
        app = new AddressBook();
        mainWindow = new ExternalWindow();
        app.setMainRootView(mainWindow.rootView());
        size = mainWindow.windowSizeForContentSize(640, 480);
        mainWindow.sizeTo(size.width, size.height);
        mainWindow.show();

        app.run();
                System.exit(0);
    }
}
