// AddressItem.java
// Copyright 1996 Netscape Communications Corp.  All rights reserved.
//
// AddressItem is a ListItem subclass which holds the data associated with
// a single address entry.  Although not a view, it is responsible for
// drawing itself in a Rect supplied by the AddressView

import netscape.application.*;
import netscape.util.*;
import java.util.StringTokenizer;
import java.io.*;

public class AddressItem extends ListItem implements TextFieldOwner,
                                                     TextFilter {
                                          // The field names
    static String    fieldNames[] = {"Name", "Street", "City", "State", "Zip",
                                     "Email", "Phone", "Fax"};
                                          // The widths of the fields
    static int       fieldWidths[] = {120, 120, 80, 50, 50, 120, 100, 100};
    static TextField fieldEditor = null;  // Shared TextField used for editing
    String           fields[];            // The name, address, etc
    int              editedField = -1;    // Field being edited, if any
    Rect             editRect;            // Location of the field editor
    int              highlightedField;    // highlighted field, if any
    String           info = "(no info)";  // Misc info on the entry


    // --------------  Constructors and initialization  --------------

    public AddressItem() {
        fields = new String[8];
        editRect = new Rect();
        highlightedField = -1;
    }

    // Returns the unique shared TextField used for editing entries
    public static TextField fieldEditor() {
        return fieldEditor;
    }

    // We need to also clone the array of strings, so that if
    // we edit one of them here, it won't change in other cloned items.
    // (New entries are created in ListView by cloning a prototype)
    public Object clone() {
        AddressItem item;

        item = (AddressItem)(super.clone());
        fields = new String[8];
        editRect = new Rect();
        highlightedField = -1;
        return item;
    }


    // -------------------  Access methods  -------------------

    // Always draw the background
    public boolean isTransparent() {
        return false;
    }

    // Required by ListItem
    public int minWidth() {
        return 750;
    }

    // Return the starting x position of the specified field
    public int fieldPosition(int fieldNum) {
        int x = 0;

        while (--fieldNum >= 0)
            x += fieldWidths[fieldNum];
        return x;
    }

    // Return the width of the specified field
    public int fieldWidth(int fieldNum) {
        return fieldWidths[fieldNum];
    }

    // Return the info string
    public String info() {
        return info;
    }

    // Set the info string
    public void setInfo(String newInfo) {
        info = newInfo;
    }

    // Return the list of fields
    public String[] fields() {
        return fields;
    }


    //  ----------------  Drawing methods  -----------------------

    // Draw the background
    protected void drawBackground(Graphics g, Rect boundsRect) {
        if (!isSelected()) {
            g.setColor(listView().backgroundColor());
            g.fillRect(boundsRect);
        }
        else {
            // If selected, draw a line at the top and bottom
            g.setColor(Color.white);
            g.fillRect(boundsRect);
            g.setColor(Color.darkGray);
            g.drawLine(boundsRect.x, boundsRect.y,
                       boundsRect.maxX(), boundsRect.y);
            g.drawLine(boundsRect.x, boundsRect.maxY() - 1,
                       boundsRect.maxX(), boundsRect.maxY() - 1);
        }
    }

    // Draw the contents
    public void drawInRect(Graphics g, Rect boundsRect) {
        Rect r = new Rect(boundsRect);
        Rect clipRect = g.clipRect();
        int  i;

        // First draw the background in the specified rectangle
        drawBackground(g, boundsRect);
        r.x = 0;
        r.width = 0;
        g.setColor(Color.black);

        // Draw the fields
        for (i = 0; i < 8; i++) {
            // Update the drawing rectangle for each entry;
            r.x += r.width;
            r.width = fieldWidths[i];

            if (i == editedField)
                continue;

            // Draw any entry that is at least partially visible
            if (r.maxX() < boundsRect.x)
                continue;

            if (r.x > boundsRect.maxX())
                break;

            g.setFont(font());

            if ((fields[i] != null) && (fields[i].length() > 0)) {
                // Draw in red if highlighted
                if (i == highlightedField)
                    g.setColor(Color.red);
                g.drawStringInRect(fields[i], r, Graphics.LEFT_JUSTIFIED);
                if (i == highlightedField)
                    g.setColor(Color.black);
            }
            else {
                // Draw the field names in gray if the field is empty
                g.setColor(Color.gray);
                g.drawStringInRect(fieldNames[i], r, Graphics.LEFT_JUSTIFIED);
                g.setColor(Color.black);
            }
        }
    }


    // ------------------  Field editing methods  ----------------

    // Begin editing at the field under the given x position
    // Superimpose the shared field editor on top of the field
    public void editAtPosition(int xPos, Rect boundsRect) {
        int x = 0;

        if (fieldEditor == null)
            fieldEditor = new TextField(editRect.x, editRect.y,
                                        editRect.width, editRect.height);
        else {
            // Complete any editing in progress elsewhere
            if (fieldEditor.isBeingEdited())
                fieldEditor.completeEditing();
        }

        editedField = 0;

        // Find the desired field based on the given x coordinate
        while (editedField < 8) {
            if (xPos < x + fieldWidths[editedField])
                break;
            x += fieldWidths[editedField];
            editedField++;
        }

        if (editedField == 8)
            return;

        // Construct a Rect for the field
        editRect.x = x;
        editRect.y = boundsRect.y;
        editRect.width = fieldWidths[editedField];
        editRect.height = boundsRect.height;

        // We want to both filter events and receive notifications
        fieldEditor.setOwner(this);
        fieldEditor.setFilter(this);
        // Place the field editor directly on top of the field
        fieldEditor.setBounds(editRect);
        fieldEditor.setTransparent(true);
        // Load the field editor with the field's text
        fieldEditor.setStringValue(fields[editedField]);
        // Display it and make it the focus of key events
        listView().addSubview(fieldEditor);
        fieldEditor.selectText();
        listView().window().setFocusedView(fieldEditor);
    }


    // -------------   TextFieldOwner interface methods  ----------------

    // Ensure that edited fields are as visible as possible
    public void textEditingDidBegin(TextField aTextField) {
        listView().scrollRectToVisible(aTextField.bounds);
    }

    // Replace the field with the newly-modified text from the field editor
    public void textEditingDidEnd(TextField aTextField,
                                  int endCondition,
                                  boolean contentsChanged) {
        if (aTextField == fieldEditor) {
            int n = editedField;

            // Take the field editor out of the view
            fieldEditor.removeFromSuperview();
            editedField = -1;

            if (n < 0)
                return;

            switch (endCondition) {
            case RESIGNED_FOCUS:
            case RETURN_KEY:
                // Return key: save the results and redraw
                fields[n] = fieldEditor.stringValue();
                listView().draw(editRect);
                break;
            case TAB_KEY:
                // Tab key: save results and go to the next field for editing
                fields[n] = fieldEditor.stringValue();
                listView().draw(editRect);
                if (n < 7)
                    editAtPosition(editRect.maxX() + 5, editRect);
                break;
            case BACKTAB_KEY:
                // BackTab key: save results and edit previous field
                fields[n] = fieldEditor.stringValue();
                listView().draw(editRect);
                if (n > 0)
                    editAtPosition(editRect.x - 5, editRect);
                break;
            case LOST_FOCUS:
                System.out.println("LOST FOCUS");
                listView().draw(editRect);
                break;
            default:
                System.out.println("DEFAULT");
                listView().draw(editRect);
                break;
            }
        }

    }

    // unused here
    public void textWasModified(TextField aTextField) {
    }

    // Unused here
    public boolean textEditingWillEnd(TextField aTextField,
                                      int endContition,
                                      boolean contentsChanged) {
        return true;
    }


    // ----------------  TextFilter method  -----------------

    // Used to look for ESC keys, which abort editing
    public boolean acceptsEvent(Object anObject, KeyEvent event,
                                Vector eventVector) {
        if (event.key == 0x1B) {
            // Trap ESC keys and abort the editing immediately
            textEditingDidEnd(fieldEditor, LOST_FOCUS, false);
            return false;
        }
        return true;
    }


    // ----------------  Incremental search methods  ----------------

    // Return the index of the field containing the searchString
    public int find(String searchString) {
        int i, j;
        int fieldLen;
        int searchLen;

        searchLen = searchString.length();

        for (i = 0; i < 8; i++) {
            if (fields[i] == null)
                continue;
            fieldLen = fields[i].length();
            for (j = 0; j < fieldLen; j++) {
                if ((fieldLen - j) < searchLen)
                    continue;
                if (fields[i].regionMatches(true, j, searchString, 0,
                                            searchLen))
                    return i;
            }
        }

        return -1;
    }

    public void highlightField(int fieldNum) {
        highlightedField = fieldNum;
    }



    // ----------------  Saving and Loading methods  -------------------

    // Write the contents to the supplied stream as a list of quoted strings
    // Note that we are storing everything as ASCII 1-byte characters instead
    // of 16-bit characters
    public void writeTo(DataOutputStream stream) throws IOException {
        int i;

        // Write all the fields, one per line, surrounded by double-quotes
        for (i = 0; i < fields.length; i++) {
            stream.writeByte('\042');
            if (fields[i] != null)
                stream.writeBytes(fields[i]);
            stream.writeBytes("\042\n");
        }

        stream.writeByte('\042');
        // Since the info can be multiple lines, we need to replace the
        // actual newlines with the "\n" string.
        if (info != null) {
            StringTokenizer tokenizer;
            String token;

            tokenizer = new StringTokenizer(info, "\n\042", true);

            while (tokenizer.hasMoreTokens()) {
                token = tokenizer.nextToken();
                if (token.equals("\n"))
                    stream.writeBytes("\\n");
                else if (token.equals("\042"))
                    stream.writeBytes("\\042");
                else
                    stream.writeBytes(token);
            }
        }
        stream.writeBytes("\042\n\n");
    }

    // Read the fields and info string from the supplied stream
    public boolean readFrom(InputStream stream) throws IOException {
        StreamTokenizer tokenizer;
        int i, type;

        // We use a Java StreamTokenizer, which can parse tokens directly
        // from the input stream.
        tokenizer = new StreamTokenizer(stream);
        tokenizer.eolIsSignificant(false);
        tokenizer.quoteChar('\042');  // indicates the use of quoted strings

        for (i = 0; i < fields.length; i++) {
            type = tokenizer.nextToken();
            if (type == StreamTokenizer.TT_EOF)
                return (i > 0);  // If we read at least one field, it's ok
            if (tokenizer.sval != null)
                fields[i] = tokenizer.sval;
        }

        // Now read the info string. The "\n"s get converted automatically
        // back to real newline characters
        type = tokenizer.nextToken();
        if (tokenizer.sval != null)
            info = tokenizer.sval;

        return true;
    }

}
