// AddressView.java
// Copyright 1996 Netscape Communications Corp.  All rights reserved.
//
// AddressView is a subclass of ListView, which manages a collection of
// ListItems.  ListItems are not Views, but "lightweight" objects representing
// a single entry in a list.  They are, however, responsible for drawing
// themselves in a Rect passed by the AddressView which owns them.
// ListView also handles selection of ListItems with the mouse.
//
// AddressView also contains save() and load() methods for reading and
// writing (when allowed) AddressItems to an ASCII file.
//

import netscape.application.*;
import netscape.util.*;
import java.io.*;
import java.util.Stack;
import java.util.EmptyStackException;

// This little local class is used to hold our current and previous
// incremental search hits
class SearchLoc {
    public int row;
    public int col;

    public SearchLoc(int row, int col) {
        this.row = row;
        this.col = col;
    }

    public SearchLoc() {
        row = col = 0;
    }
}

public class AddressView extends ListView implements Target, TextFilter {
    Point        clickPoint;              // Stores the last mouse click point
    StringBuffer searchBuffer;            // Stores characters for searching
    TextView     infoEditor;              // Displays the info text
    AddressItem  infoItem;                // Source of current info text
    Stack        searchStack;             // A stack of search hits

    // --------------  Constructors and initialization  --------------

    public AddressView(int x, int y, int width, int height) {
        super(x, y, width, height);
        clickPoint = new Point();
        searchStack = new Stack();        // Used for searches

        // Our ListItem prototype will be an AddressItem instance
        setPrototypeItem(new AddressItem());
        prototypeItem().setTitle("New Address Entry");

        // Allow selection of more than one AddressItem at a time
        setAllowsMultipleSelection(true);
        // No selection is ok, too
        setAllowsEmptySelection(true);

        // When user double clicks in the view, call performCommand with
        // "Edit" as the command argument
        setDoubleCommand("edit");
        setTarget(this);

        // This is used to accumulate characters from the search TextField
        searchBuffer = new StringBuffer();
    }

    // AddressBook tells us where to display the info for a selected entry
    public void setInfoEditor(TextView view) {
        infoEditor = view;
    }


    // --------------  Entry Creation and Deletion methods  -------------

    // Called by AddressBook to create a new entry
    public void newItem() {
        ListItem item;
        Rect     r;
        TextField fieldEditor = AddressItem.fieldEditor();

        if ((fieldEditor != null) && fieldEditor.isBeingEdited())
            fieldEditor.completeEditing();

        // Create the new item (at the end)
        item = addItem();

        // Resize to include the new item
        sizeToMinSize();

        // Get the drawing rectangle of the new item
        r = rectForItem(item);
        // Set the width to a small number to insure we're scrolled hard left
        r.width = 10;
        scrollRectToVisible(r);

        // Now update the display
        draw();
    }

    // Called by AddressBook to delete one or more entries
    public void delete() {
        Vector selections;
        int    i, n;
        TextField fieldEditor = AddressItem.fieldEditor();

        if ((fieldEditor != null) && fieldEditor.isBeingEdited())
            fieldEditor.completeEditing();

        // We must clone the selection vector because the original is modified
        // by the removeItem calls
        selections = (Vector)(selectedItems().clone());
        n = selections.count();

        if (n > 0) {
            for (i = 0; i < n; i++)
                removeItem((ListItem)(selections.elementAt(i)));
            // Resize to fit the remaining items
            sizeToMinSize();
            // Update the display
            draw();
            clearInfo();
        }
    }

    void clearInfo() {
        infoEditor.setString("");
        infoEditor.setBounds(infoEditor.superview().bounds);
    }

    // ------------  Mouse event handling methods  -----------------

    // If an entry is being edited, finish the editing
    // Save the mouse position in case this becomes a doubleClick
    // event and the field editor has to be moved
    public boolean mouseDown(MouseEvent event) {
        TextField fieldEditor = AddressItem.fieldEditor();

        clickPoint.moveTo(event.x, event.y);
        if ((fieldEditor != null) && fieldEditor.isBeingEdited())
            fieldEditor.completeEditing();
        return super.mouseDown(event);
    }

    // If there are multiple or no selections, clear the info TextView.
    // Otherwise display the info from the newly-selected entry
    public void mouseUp(MouseEvent event) {
        boolean multipleSelections = selectedItems().count() > 1;

        super.mouseUp(event);

        if (multipleSelections || (selectedItem() == null)) {
            if (infoItem != null)
                infoItem.setInfo(infoEditor.string());
            clearInfo();
            infoItem = null;
        }
        else if (infoItem != selectedItem()) {
            if (infoItem != null)
                infoItem.setInfo(infoEditor.string());
            infoItem = (AddressItem)selectedItem();
            if (infoItem.info() != null) {
                infoEditor.setString(infoItem.info());
            }
            else
                clearInfo();
        }
    }

    // Start editing a row
    public void edit(Object anObject) {
	AddressItem item = (AddressItem)selectedItem();
	if (item != null)
	    item.editAtPosition(clickPoint.x, rectForItem(item));
    }


    // -----------------  Target interface method  ---------------

    // This is generated by a double-click because we called
    // setDoubleCommand() above and also set ourselves as the target.
    // Find the entry that was clicked on, and pass it its Rect
    // and also the mouse click position (so it can tell which field
    // was clicked on)
    public void performCommand(String command, Object anObject) {
        if (command != null && command.equals("edit")) {
	    edit(anObject);
        }
    }


    // ----------------  Incremental Search methods  -----------------

    // Starting at the previous search position, search forward to
    // find a match with the current search string.
    private void searchForward() {
        String      searchString = searchBuffer.toString();
        AddressItem item = null;
        SearchLoc   loc;
        int         row;

        try {
            loc = (SearchLoc)(searchStack.peek());
            item = (AddressItem)itemAt(loc.row);
            if (item != null) {
                // Turn off any highlighting
                item.highlightField(-1);
                drawItemAt(loc.row);
            }
            loc = new SearchLoc(loc.row, loc.col);
        }
        catch (EmptyStackException e) {
            loc =  new SearchLoc();
            item = (AddressItem)itemAt(0);
        }

        while ((item != null) && (loc.row < count())) {
            loc.col = item.find(searchString);
            if (loc.col > -1) {
                // Found it
                Rect r = rectForItemAt(loc.row);
                r.x = item.fieldPosition(loc.col);
                r.width = item.fieldWidth(loc.col);
                // Highlight the field
                item.highlightField(loc.col);
                // Make sure it's visible
                scrollRectToVisible(r);
                draw(r);
                // Save the location on the stack
                searchStack.push(loc);
                break;
            }
            loc.row++;
            item = (AddressItem)itemAt(loc.row);
        }
    }

    // Not really a backward search, just a reversion to the previous
    // search hit
    private void searchBackward() {
        SearchLoc   loc;
        AddressItem item;

        try {
            // Pop the current search location off the stack
            loc = (SearchLoc)(searchStack.pop());
            item = (AddressItem)itemAt(loc.row);
            if (item != null) {
                // Turn off any highlighting
                item.highlightField(-1);
                drawItemAt(loc.row);
            }
        }
        catch (EmptyStackException e) {
            // We're back at the beginning
            return;
        }

        // Highlight the previous search location
        try {
            Rect r;

            loc = (SearchLoc)(searchStack.peek());
            r = rectForItemAt(loc.row);
            item = (AddressItem)itemAt(loc.row);
            r.x = item.fieldPosition(loc.col);
            r.width = item.fieldWidth(loc.col);
            // Highlight the field
            item.highlightField(loc.col);
            // Make sure it's visible
            scrollRectToVisible(r);
            draw(r);
        }
        catch (EmptyStackException e) {
        }
    }


    // ----------------  TextFilter interface method  ---------------

    // This is the TextFilter interface method which is called each time
    // a new key is typed into the search field
    public boolean acceptsEvent(Object textObject, KeyEvent event,
                                Vector events) {
        // Only allow editing at the end of the text in this case
        ((TextField)textObject).setInsertionPoint(999);
        if (event.isArrowKey() || (event.key == 127))
            return true;
        if (event.isBackspaceKey()) {
            // Backspace, so lop a character off the end and backup the search
            if (searchBuffer.length() > 0)
                searchBuffer.setLength(searchBuffer.length() - 1);
            searchBackward();
            return true;
        }
        else if (event.isPrintableKey()) {
            searchBuffer.append((char)(event.key));
            searchForward();
        }
        return true;
    }


    // --------------  File saving and loading methods  ----------------

    // Write all the entries to the supplied OutputStream
    public void save(OutputStream stream) {
        int      i = 0, n;
        DataOutputStream dataStream;
        TextField fieldEditor = AddressItem.fieldEditor();

        // Make sure the results of any editing get stored
        if ((fieldEditor != null) && fieldEditor.isBeingEdited())
            fieldEditor.completeEditing();
        if (infoItem != null)
            infoItem.setInfo(infoEditor.string());

        dataStream = new DataOutputStream(stream);
        n = count();
        try {
            for (i = 0; i < n; i++)
                ((AddressItem)itemAt(i)).writeTo(dataStream);
        }
        catch (IOException e) {
	    Alert.runAlertInternally(Alert.warningImage(),"Save",
				     "I/O Error while saving row " + (i + 1) +
				     ": " + e,"OK",null,null);
        }

    }

    // Load entries from the specified InputStream
    public void load(InputStream stream) {
        AddressItem item = null;
        int n = 0;

        // First clear all existing entries
        removeAllItems();

        try {
            while (stream.available() > 0) {
                item = (AddressItem)addItem();  // Create a new entry
                if (!item.readFrom(stream)) {  // Try to read it
                    removeItemAt(n);           // Remove it if an error or EOF
                    break;
                }
                n++;
            }
        }
        catch (IOException e) {
            if (item != null)
                removeItem(item);
	    Alert.runAlertInternally(Alert.warningImage(),"Open",
				     "Error while reading row " + (n + 1) + 
				     ": " + e ,"OK",null,null);
        }

        // Resize ourself to fit the new number of entries
        sizeToMinSize();
        draw();
    }

}




