// FileViewer.java
// Copyright 1996 Netscape Communications Corp.  All rights reserved.
//
// A subclass of InternalWindow set up to display file choices.
// Displays a list of files from the indicated directory, relative to the
// application's code base, whether local or over the net.  Allows
// filtering by extension.
//
// Files are displayed as an array of FileButtons, a Button class which
// displays the file name under a document icon image.  A TextField
// at the bottom displays the current selection, or allows typing in
// a new name (i.e., for saving).
//
// When a FileButton is double-clicked, or the return key is hit in the
// TextField, a user-specified command is sent to the target.  The target can
// then request the full URL of the selected file.
//
// Could be easily extended to display directory icons, to allow moving up
// and down in the directory hierarchy.

import netscape.application.*;
import netscape.util.*;
import java.io.*;
import java.net.*;
import java.util.StringTokenizer;

public class FileViewer extends InternalWindow implements FilenameFilter {
    String      directory;                // Directory being displayed
    String      fileName;                 // Name of selected file
    String      extension;                // Extension of displayed files
    FileButton  icons[];                  // Current list of FileButtons
    TextField   field;                    // Displays name of selected file
    ScrollGroup scrollGroup;              // Used to scroll icon display
    Target      target;                   // Target of file selection
    String      command;                  // Sent when file is selected
    String      codeBase;                 // Our application's code base
    String      codeDir;                  // The directory of the code base
    String      protocol;                 // Our protocol

    // --------------  Constructors and initialization  --------------

    public FileViewer(int x, int y, int width, int height) {
        super(x, y, width, height);

        Size          iSize;              // Window's interior size
        ContainerView view;               // Holds the file icons

        codeBase = "";
        protocol = "";
        directory = "";
        fileName = "";
        extension = null;
        setResizable(true);
        setCloseable(true);
        setCanBecomeDocument(false);
        setTitle("Select File");

        iSize = contentSize();

        // Create and place the field which displays the selected file's name
        field = new TextField(0, iSize.height - 20, iSize.width, 20);
        field.setFont(new Font("Helvetica", Font.PLAIN, 14));
        field.setHorizResizeInstruction(WIDTH_CAN_CHANGE);
        field.setVertResizeInstruction(TOP_MARGIN_CAN_CHANGE);
        field.setTarget(this);
        field.setCommand("fileSelection");
        addSubview(field);

        // A ScrollGroup is used to handle scrolling of file icons
        scrollGroup = new ScrollGroup(0, 0, iSize.width,
                                      iSize.height - field.bounds.height);
        scrollGroup.setBackgroundColor(Color.lightGray);
        scrollGroup.setBorder(new BezelBorder(BezelBorder.LOWERED,
                                              Color.lightGray));
        view = new ContainerView();
        view.setBackgroundColor(Color.lightGray);
        scrollGroup.setContentView(view);
        addSubview(scrollGroup);
    }

    // Set the required extension.  We only display files ending with this
    // extension.
    public void setRequiredExtension(String extension) {
        // Make sure it starts with a dot
        this.extension = (extension.charAt(0) == '.') ? extension :
            "." + extension;
    }

    // Set the object to be notified when a file is selected
    public void setTarget(Target anObject) {
        target = anObject;
    }

    // Set the command to be sent when a file is selected
    public void setCommand(String aCommand) {
        command = aCommand;
    }


    // ------------------  Access methods  ---------------------

    // Returns the directory of the selected file
    public String directory() {
        return directory;
    }

    // Returns the name of the selected file
    public String fileName() {
        return fileName;
    }

    // Construct and return the full URL of thecurrently selected file
    public URL fileURL () {
        URL url = null;
        String urlstr;

        // If the code base includes no protocol, assume the file protocol
        if (!(codeBase.startsWith("http") || codeBase.startsWith("file") ||
              codeBase.startsWith("HTTP") || codeBase.startsWith("FILE")))
            urlstr = (protocol != null) ? (protocol + ":") : "file:";
        else
            urlstr = "";

        // Add the code base and the selection directory
        urlstr += codeBase + directory;

        // Add a / at the end if there isn't one already
        if (directory.length() > 0) {
            if (!directory.endsWith("/"))
                urlstr += "/";
        } else {
            if (!codeBase.endsWith("/"))
                urlstr += "/";
        }

        // Finally, add the filename
        urlstr += fileName;

        // Construct a URL instance from the full string
        try {
            url = new URL(urlstr);
        }
        catch (MalformedURLException e) {
	    Alert.runAlertInternally(Alert.warningImage(),"FileViewer",
				     "Problem constructing URL: " + e,
				     "OK",null,null);
        }

        return url;
    }

    // Construct and return the URL of the currently displayed directory
    public URL dirURL () {
        URL url = null;
        String urlstr;

        // If the code base includes no protocol, assume the file protocol
        if (!(codeBase.startsWith("http") || codeBase.startsWith("file") ||
              codeBase.startsWith("HTTP") || codeBase.startsWith("FILE")))
            urlstr = (protocol != null) ? (protocol + ":") : "file:";
        else
            urlstr = "";

        // Add the code base and the selection directory
        urlstr += codeBase + directory;

        // Construct a URL instance from the full string
        try {
            url = new URL(urlstr);
        }
        catch (MalformedURLException e) {
	    Alert.runAlertInternally(Alert.warningImage(),"FileViewer",
				     "Problem constructing URL: " + e,
				     "OK",null,null);
        }

        return url;
    }


    // --------------  Directory listing methods  ----------------

    // Use the File class's list() method to get a list of files from
    //  the local file system
    public String[] getLocalFileNames() {
        File dir;
        String list[] = null;
        String path = codeDir;

        if ((directory != null) && (directory.length() > 0)) {
            if (path.endsWith("/"))
                path += directory;
            else
                path += ("/" + directory);
        }

        try {
            dir = new File(path);
            if (dir != null)
                list = dir.list(this);    // Use our accept() method
        }
        catch (SecurityException e) {
            return null;
        }

        return list;
    }

    // Get a directory index from the remote http server, and return a list
    // of all HREF file references.
    public String[] getRemoteFileNames() {
        String list[] = null;
        DataInputStream stream = null;
        int n;
        StringBuffer buffer;
        String s, token;
        StringTokenizer tokenizer;
        Vector hrefs;

        // Load the remote directory index into a StringBuffer instance
        try {
            stream = new DataInputStream(dirURL().openStream());
            buffer = new StringBuffer();
            while (true) {
                s = stream.readLine();
                if (s == null)
                    break;
                buffer.append(s);
            }
            stream.close();
        }
        catch (IOException e) {
	    Alert.runAlertInternally(Alert.warningImage(),"FileViewer",
				     "Problem reading remote directory " 
				     + dirURL() + ": " + e,
				     "OK",null,null);
            return null;
        }

        // Set up a StringTokenizer to break on whitespace, "=", and
        // double quotes
        tokenizer = new StringTokenizer(buffer.toString(), " \t\n\r=\042");
        hrefs = new Vector();

        // Look for HREFs and store the indicated file names
        while (tokenizer.hasMoreTokens()) {
            token = tokenizer.nextToken();
            if (token.equalsIgnoreCase("HREF")) {
                token = tokenizer.nextToken();  // The file name
                if (token == null)
                    continue;
                if (accept(null, token)) {
                    token = removeCodeBase(token); // Strip the directory
                    hrefs.addElement(token);
                }
            }
        }

        // Convert the file name vector into a list of strings
        list = new String[hrefs.count()];
        hrefs.copyInto(list);

        return list;
    }


    // -----------  File icon construction and layout methods ---------

    // When the window is made visible, display a list of file names from the
    // specified directory (relative to the code base), if any.
    public void showModally() {
        String list[] = null;
        int i, x, y;
        FileButton icon = null;
        int width, height;
        View view = scrollGroup.contentView();

        clear();

        // Set the code base, the directory from which the applet was loaded
        // or from which the application was launched
        if (Application.application() != null)
            setCodeBase(Application.application().codeBase());

        if (codeBase.length() == 0) {
            return;
	}

        // a File protocol means we're running locally from appletviewer
        // or stand-alone
        if (protocol.equals("file"))
            list = getLocalFileNames();

        // http protocol means we've arrived from the net
        if ((list == null) || protocol.startsWith("http"))
            list = getRemoteFileNames();

        if (list == null) {
            return;
	}

        // Estimate a reasonable width for the viewer, 150 times the
        // square root of the number of files
        width = 150 * (int)Math.max((int)Math.sqrt((double)list.length), 1);

        x = 50;
        y = 10;
        icons = new FileButton[list.length];

        // Construct and layout the FileButton icons 100 pixels from center
        // to center horizontally and separated by 10 pixels vertically
        for (i = 0; i < list.length; i++) {
            icon = new FileButton(list[i]);
            icon.setTarget(this);
            icon.setCommand("fileSelection");
            if ((x + icon.bounds.width / 2) > width) {
                y += icon.bounds.height + 10;
                x = 50;
            }
            icon.moveTo(x - icon.bounds.width / 2, y);
            view.addSubview(icon);
            icons[i] = icon;
            x += 100;
        }

        height = y + ((icon != null) ? icon.bounds.height : 0);

        view.sizeTo(Math.max(width, scrollGroup.scrollView().bounds.width),
                    Math.max(height, scrollGroup.scrollView().bounds.height));

        // Add scroll bars only if needed
        if (view.bounds.width > scrollGroup.bounds.width)
            scrollGroup.setHasHorizScrollBar(true);
        if (view.bounds.height > scrollGroup.bounds.height)
            scrollGroup.setHasVertScrollBar(true);

        super.showModally();
    }


    // Select an item. If the anObject is the textfield
    // or if the user did doubleclick, terminate the session
    public void fileSelection(Object anObject) {
	if (anObject == field)
	    fileSelected(field.stringValue());
	else if (((FileButton)anObject).clickCount() == 2)
	    fileSelected(((FileButton)anObject).title());
	else
	    iconSelected((FileButton)anObject);	
    }

    // ------------  Target interface and related methods  -----------

    // Single clicks select icons within the viewer, double-clicks indicate
    // a final selection
    public void performCommand(String command, Object anObject) {
        if (command.equals("fileSelection")) {
	    fileSelection(anObject);
        } else {
            super.performCommand(command, anObject);
        }
    }



    // When an icon is selected, copy the file name to the editing field
    // and deselect any previously-selected icons
    public void iconSelected(FileButton icon) {
        int i;

        if (!icon.state())
            icon.setState(true);
        field.setStringValue(icon.title());
        for (i = 0; i < icons.length; i++)
            if (icons[i] != icon)
                icons[i].setState(false);
    }

    // Indicate to the target that a file was selected by a double-click on
    // a FileButton or a return in the TextField, and close up
    public void fileSelected(String name) {
        fileName = name;
        if (target != null)
            target.performCommand(command, this);
        hide();
    }


    // ---------------------  Other methods  -------------------

    // Set the code base, that is, the URL of the main file or directory
    // where the applet's code was loaded from
    public void setCodeBase(URL base) {
        codeBase = base.toString();
        protocol = base.getProtocol();
        if (!codeBase.endsWith("/"))
            // If there's a file name on the end, lop it off
            codeBase = codeBase.substring(0, codeBase.lastIndexOf('/') + 1);
        if (!(codeBase.startsWith("http") || codeBase.startsWith("file") ||
              codeBase.startsWith("HTTP") || codeBase.startsWith("FILE")))
            // If there's no protocol, the directory is the same as the base
            codeDir = codeBase;

        else
            // Otherwise, the directory follows the protocol
            codeDir = codeBase.substring(codeBase.indexOf(':') + 1,
                                         codeBase.length());
    }

    // Clear the view of any existing FileButton views
    public void clear() {
        int i, n;
        Vector icons = (Vector)(scrollGroup.contentView().subviews().clone());
        n = icons.count();
        for (i = 0; i < n; i++)
           ((View)(icons.elementAt(i))).removeFromSuperview();
    }

    // Strip the protocol and directory from the given full file path
    String removeCodeBase(String file) {
        int i;

        i = file.lastIndexOf('/');            // Unix
        if (i >= 0)
            file = file.substring(i + 1, file.length());
        else {
            i = file.lastIndexOf('\\');       // Windoze
            if (i >= 0)
                file = file.substring(i + 1, file.length());
        }

        return file;
    }
    // This is from the FilenameFilter interface. This method is called by
    // the File class's list() method to determine which files we want to
    // see in the returned listing.
    public boolean accept(File dir, String name) {
        // If an extension is specified, only allow files that end in
        // the extension
        return (extension == null) || (extension.equals("")) ||
               name.endsWith(extension);
    }

    // The viewer cannot be made smaller than this
    public Size minSize() {
        return new Size(280, 160);
    }

}

