// Aquarium.java
// By Ned Etcode
// Copyright 1995, 1996 Netscape Communications Corp.  All rights reserved.

import netscape.application.*;
import netscape.util.*;
import java.io.*;
import java.net.*;

/** In this example we create an aquarium tank, and a pallete of supplies.
  * Drag and drop the animated fish, bubbles, and plants into the aquarium,
  * Where they animate themselves using DrawingSequences.
  *
  * This example primarily illustrates the use of the following classes:
  * <ul>
  *    <li>DragSession
  *    <li>DrawingSequence
  * </ul>
  * Secondarily, it also illustrates:
  * <ul>
  *    <li>Mouse events in View
  *    <li>Mouse Tracking Events in View
  *    <li>Bitmaps
  * <ul>
  * There are four items on the palette. Each of the four views displays
  * an icon, initiates a drag session when the user clicks and drags the
  * mouse over it, and supplies an instance of its related DrawingSequence
  * when dropped onto the tank.
  *
  * The tank receives dropped items, requests a DrawingSequence from the
  * item, sets itself as the sequence's owner, and starts its animation.
  */

public class Aquarium extends Application implements Target, WindowOwner {
    Button            onOffSwitch;
    InternalWindow    supplies;          // The supplies palette window
    InternalWindow    invoice;
    ScrollGroup       purchaseScroll;
    ListView          purchaseList;
    Vector            products;

    public void init() {
        int            x, width, height;
        ContainerView  container;
        TankView       tankView;          // The aquarium tank view
        IconView       icon;
        RootView       rootView = mainRootView();
        InternalWindow controls;
        PurchaseItem   purchaseItem;
        Bitmap image;

        super.init();

        products = readVector("products");

        // The tank should be exactly the size of the background
        tankView = new TankView(0, 0, rootView.width(), rootView.height());
        rootView.addSubview(tankView);

        supplies = new InternalWindow(410, 10, 180, 150);
        supplies.setRootView(rootView);
        supplies.setTitle("Aquarium Supplies");

        width = supplies.contentView().width();
        height = supplies.contentView().height();
        container = new ContainerView(-1, -1, width, height);
        container.setImageDisplayStyle(Image.TILED);
        container.setBorder(null);
        container.setImage(Bitmap.bitmapNamed("WaterTile.gif"));

        // Create, position, and add icons for each supply palette item
        x = 10;

        // Add an icon for each of the two kinds of "fish"
        icon = new FishIcon(x, 70, "fishE", 5);
        icon.info = (Hashtable)products.elementAt(3);
        container.addSubview(icon);
        icon = new FishIcon(x, 10, "fishA", 6);
        icon.info = (Hashtable)products.elementAt(2);
        container.addSubview(icon);

        // Add an icon for the plants
        x += icon.width() + 15;
        icon = new PlantIcon(x, 10);
        icon.info = (Hashtable)products.elementAt(1);
        container.addSubview(icon);

        // Add an icon for the bubbles
        x += icon.width() + 15;
        icon = new BubbleIcon(x, 10);
        icon.info = (Hashtable)products.elementAt(0);
        container.addSubview(icon);


        // Show the supplies
        supplies.addSubview(container);
        supplies.setCloseable(true);
        supplies.show();

        image = Bitmap.bitmapNamed("ToggleOff.gif");
        controls = new InternalWindow(Window.BLANK_TYPE, 21, 17,
	                              image.width(), image.height());
        controls.setRootView(rootView);
        controls.setTransparent(false);
        controls.setLayer(500);

        onOffSwitch = new Button(0, 0, image.width(), image.height());
        onOffSwitch.setType(Button.TOGGLE_TYPE);
        onOffSwitch.setImage(image);
        onOffSwitch.setBuffered(true);
        image = Bitmap.bitmapNamed("ToggleOn.gif");
        onOffSwitch.setAltImage(image);
        onOffSwitch.setTarget(tankView);
        onOffSwitch.setCommand("toggleSwitch");
        onOffSwitch.setTransparent(true);

        controls.addSubview(onOffSwitch);
	controls.setLayer(InternalWindow.DEFAULT_LAYER - 1);
        controls.show();

        invoice = new InternalWindow(10, 200, 200, 150);
        invoice.setRootView(rootView);
        invoice.setTitle("Invoice");
        invoice.setCloseable(true);
        width = invoice.contentView().width();
        height = invoice.contentView().height();
        purchaseScroll = new ScrollGroup(8, 8, width - 16, height - 16);
        purchaseScroll.setBorder(BezelBorder.loweredBezel());
        purchaseScroll.setHasVertScrollBar(true);
        purchaseList = new ListView(0, 0,
	                            purchaseScroll.scrollView().width(), 0);
        purchaseList.setRowHeight(24);
        purchaseItem = new PurchaseItem();
        purchaseItem.setFont(Font.fontNamed("Helvetica", Font.PLAIN, 18));
        purchaseList.setPrototypeItem(purchaseItem);
        purchaseScroll.setContentView(purchaseList);
        purchaseScroll.scrollView().setBuffered(true);
        invoice.addSubview(purchaseScroll);
    }

    Vector readVector(String fileName) {
        URL vectorURL;
        InputStream inputStream;
        Vector vector;

        try {
            vectorURL = new URL(codeBase(), fileName);
            inputStream = vectorURL.openStream();
            vector = (Vector)Deserializer.readObject(inputStream);
            inputStream.close();
        } catch (IOException e) {
            vector = null;
        }
        return vector;
    }

    public void performCommand(String command, Object data) {
        if ("showSupplies".equals(command)) {
            showSupplies();
        } else if ("showInvoice".equals(command)) {
            showInvoice();
        }
    }

    void purchaseItem(Hashtable productInfo) {
        PurchaseItem purchaseItem = (PurchaseItem)purchaseList.addItem();

        purchaseItem.setProductInfo(productInfo);
        purchaseList.sizeToMinSize();
        purchaseList.scrollItemAtToVisible(purchaseList.count() - 1);
        purchaseList.selectItemAt(purchaseList.count() - 1);
        purchaseList.draw();
    }

    void showSupplies() {
        supplies.show();
    }

    void showInvoice() {
        invoice.show();
    }

    public boolean windowWillShow(Window aWindow) {
	return true;
    }

    public void windowDidShow(Window aWindow) {
    }

    public boolean windowWillHide(Window aWindow) {
	return true;
    }

    public void windowDidHide(Window aWindow) {
	stopRunning();
    }

    public void windowDidBecomeMain(Window aWindow) {
    }

    public void windowDidResignMain(Window aWindow) {
    }

    public void windowWillSizeBy(Window aWindow, Size deltaSize) {
    }
    
    public static void main(String args[]) {
        Aquarium app;
        ExternalWindow mainWindow;
        Size size;

        app = new Aquarium();
        mainWindow = new ExternalWindow();
	mainWindow.setTitle("The Wanda Fish Company");
	mainWindow.setOwner(app);
	mainWindow.setResizable(false);
        size = mainWindow.windowSizeForContentSize(640, 410);
        mainWindow.sizeTo(size.width, size.height);
        mainWindow.show();
        app.setMainRootView(mainWindow.rootView());

        app.run();
	System.exit(0);
    }
}
