// Fish.java
// By Ned Etcode
// Copyright 1995, 1996 Netscape Communications Corp.  All rights reserved.

import netscape.application.*;
import netscape.util.*;
import java.util.Random;

/** This DrawingSequence handles animation of the fish. It loads two arrays
  * of Bitmaps. One array has images of right-facing images, and the other
  * has left-facing images. We switch between them depending on which
  * direction we're going. Normally we would use an ImageSequence to animate
  * a sequence of Bitmaps, but because of the left-right switches, we do
  * it ourselves. Implementing the Movable interface adds a notion of
  * position and bounds. We override the nextFrame() method in order to
  * adjust our position for each frame. The owner (TankView) then draws us
  * at our current position.
  */
public class Fish extends DrawingSequence implements Movable {
    int    maxWidth;                      // Our largest width
    int    maxHeight;                     // Our largest height
    int    xDirection;                    // Controls left vs. right movement
    int    yDirection;                    // Controls up vs. down movement
    Rect   tankBounds;                    // The bounds of the fish tank
    Rect   currentBounds;                 // Current drawing area
    Rect   lastBounds;                    // Previous drawing area
    Bitmap imagesRight[];                 // Right-facing images
    Bitmap imagesLeft[];                  // Left-facing images

    // Randomizer to get fish speed.
    static Random randomizer = new Random();

    public Fish(String prefix, int numImages) {
        super(null);

        int i;

        xDirection = 1;                   // Start out by going right and down
        yDirection = 1;

        // Load all the bitmaps, whose names are derived from the supplied
        // prefix
        imagesRight = new Bitmap[numImages];
        imagesLeft = new Bitmap[numImages];

        for (i = 0; i < numImages; i++) {
            imagesLeft[i] = Bitmap.bitmapNamed(prefix + (i + 1) + ".gif");
            imagesRight[i] = Bitmap.bitmapNamed(prefix + (i + 1) + "r.gif");

            // Remember the maxWidth and maxHeight for all the frames
            if (imagesLeft[i].width() > maxWidth)
                maxWidth = imagesLeft[i].width();
            if (imagesLeft[i].height() > maxHeight)
                maxHeight = imagesLeft[i].height();
            if (imagesRight[i].width() > maxWidth)
                maxWidth = imagesRight[i].width();
            if (imagesRight[i].height() > maxHeight)
                maxHeight = imagesRight[i].height();
        }

        // Initialize the bounds
        currentBounds = new Rect(0, 0, maxWidth, maxHeight);
        lastBounds = new Rect(currentBounds);

        setPlaybackMode(FORWARD_LOOP);

        setFrameRate(50 + (randomizer.nextInt() % 50));
//        setFrameRate(150);                // in milliseconds
        setFrameCount(numImages);
    }

    /** Reset the current drawing position
      */
    public void setPosition(int x, int y)  {
        currentBounds.x = x;
        currentBounds.y = y;
    }

    /** Reset the current movement limits
      */
    public void setTankBounds(int x, int y, int width, int height) {
        tankBounds = new Rect(x, y, width, height);
    }

    public Rect currentBounds() {
        return currentBounds;
    }

    public Rect lastBounds() {
        return lastBounds;
    }

    /** This gets called once per frame. We override it in order to adjust
      * our location before the frame gets drawn by the sequence's owner.
      */
    public boolean nextFrame() {
        boolean returnVal;
        int     i, speedFactor;
        int     frameRate;

        // Save the previously drawn bounds
        lastBounds.setBounds(currentBounds);

        // This causes the fish to move faster when it is "stretched out"
        // (i.e., higher frame numbers)
        speedFactor = (currentFrameNumber() + 1) * 2;

        // This adjustment makes it look better when moving to the left
        if (xDirection == -1)
            speedFactor *= 1.5;

        // Set the new position
        currentBounds.x +=  speedFactor * xDirection;
        currentBounds.y += 2 * yDirection;

        // When we hit the left or right edge of our moving area,
        // change direction and swap the image lists.
        if (currentBounds.maxX() > tankBounds.maxX()) {
            currentBounds.x = tankBounds.maxX() - width();
            xDirection = -1;
        } else if (currentBounds.x < tankBounds.x) {
            currentBounds.x = tankBounds.x;
            xDirection = 1;
        }

        // Change direction if we hit the bottom or top of the range
        if (currentBounds.maxY() > tankBounds.maxY()) {
            currentBounds.y = tankBounds.maxY() - height();
            yDirection = -1;
        } else if (currentBounds.y < tankBounds.y) {
            currentBounds.y = tankBounds.y;
            yDirection = 1;
        }

        // This will change the frame number and ask our owner view to draw us
        returnVal = super.nextFrame();

        // Cycle up and down the image list.
        if ((playbackMode() == FORWARD_LOOP) &&
            (currentFrameNumber() == frameCount() - 1)) {
            setPlaybackMode(BACKWARD_LOOP);
        } else if ((playbackMode() == BACKWARD_LOOP) &&
                 (currentFrameNumber() == 0)) {
            setPlaybackMode(FORWARD_LOOP);
        }
        return returnVal;
    }

    public Image currentImage() {
        Image currentImage;

        if (xDirection == 1)
            currentImage = imagesRight[currentFrameNumber()];
        else
            currentImage = imagesLeft[currentFrameNumber()];

        return currentImage;
    }

    public int width() {
        return maxWidth;
    }

    public int height() {
        return maxHeight;
    }

    public void drawAt(Graphics g, int x, int y) {
        // To the outside, a Fish appears to have constant width and height,
        // but each frame may be a different size so we draw them centered
        // in our bounds.
        currentImage().drawCentered(g, x, y, width(), height());
    }

    public void drawScaled(Graphics g, int x, int y, int width, int height) {
        // Our fish don't scale (get it?)
        drawCentered(g, x, y, width, height);
    }
}
