// Plant.java
// By Ned Etcode
// Copyright 1995, 1996 Netscape Communications Corp.  All rights reserved.

import netscape.application.*;
import netscape.util.*;

/** This DrawingSequence handles animation of the plants. The plants don't
  * move, but they do wave around like kelp within their fixed bounds. Each
  * plant consists of three wiggly lines, stored in a Polygon instance. The
  * polygons are created as soon as we know our bounds, then adjusted when
  * drawn to lean back and forth according to the frame number.
  */
public class Plant extends DrawingSequence implements Movable {
    Rect    currentBounds;                // Our drawing bounds
    Rect    lastBounds;                   // Not actually needed here.
    Polygon polygons[];                   // The plant polygons
    Polygon drawPolygons[];               // Same, adjusted for "waving"

    public Plant() {
        super(null);

        currentBounds = new Rect(0, 0, 0, 0);
        lastBounds = new Rect(currentBounds);

        // These polygons hold the lines representing the 3 plant strands
        polygons = new Polygon[3];
        polygons[0] = new Polygon();
        polygons[1] = new Polygon();
        polygons[2] = new Polygon();

        // These are copies of the above, adjusted to lean left or right
        drawPolygons = new Polygon[3];

        setPlaybackMode(FORWARD_LOOP);
        setFrameRate(200);                // in milliseconds
        setFrameCount(30);                // arbitrary
    }

    /** Reset the current drawing position
      */
    public void setPosition(int x, int y) {
        currentBounds.x = x;
        currentBounds.y = y;
        lastBounds.setBounds(currentBounds);
    }

    /** Reset the current moving range. Now that we know our bounds, we can
      * create the Polygons.
      */
    public void setTankBounds(int x, int y, int width, int height) {
        int    i, j, n, dy;
        double dx, r;

        currentBounds.width = 40;
        currentBounds.height = height - currentBounds.y - 10;
        lastBounds.setBounds(currentBounds);

        n = frameCount();
        dx = currentBounds.width / 2;
        dy = currentBounds.height / 10;
        x = currentBounds.x + currentBounds.width / 2;

        // Start at the bottom and add a gradually increasing randomness
        // at each point, in the x dimension
        for (i = 0; i < 3; i++) {
            for (j = 0; j < 10; j++) {
                r = 1.0 - Math.random() * 2.0;
                polygons[i].addPoint(x + (int)(dx * r * (double)j / 9.0),
                                     currentBounds.y + currentBounds.height -
                                     10 - dy * j);
            }

            // Copy each polygon into the corresponding drawPolygon
            drawPolygons[i] = new Polygon(polygons[i].xPoints,
                                          polygons[i].yPoints, 10);
        }
    }

    public Rect currentBounds() {
        return currentBounds;
    }

    public Rect lastBounds() {
        return lastBounds;
    }

    public int width() {
        return currentBounds.width;
    }

    public int height() {
        return currentBounds.height;
    }

    Color randomGreen() {
        int r = 0;
        int g = 75 + (int)(Math.random() * 75.0);
        int b = g - (int)(Math.random() * 75.0);

        return new Color(r, g, b);
    }

    /** Since our drawing bounds never changes, we can calculate our polygons
      * here instead of having to also override nextFrame().
      */
    public void drawAt(Graphics g, int x, int y) {
        int   i, j, n, frame;
        float waveFactor;
        int   nFrames = frameCount();

        n = polygons[0].numPoints;

        for (i = 0; i < 3; i++) {
            // offset each polyon's frame number
            frame = currentFrameNumber() + i * 5;
            if (frame > nFrames)
                frame -= nFrames;

            // Convert to a triangle wave pattern (up, down, up, etc)
            if (frame > (nFrames / 2))
                frame = nFrames - frame;

            // Compute a back-and-forth offset factor between -1 and 1
            waveFactor = ((float)frame - (float)nFrames * 0.25f) /
                ((float)nFrames * 0.25f);

            // Add the offset to the x positions.  Add more as we get higher.
            for (j = 1; j < n; j++)
                drawPolygons[i].xPoints[j] =
                    polygons[i].xPoints[j] + (int)((float)j * waveFactor);
        }

        g.setColor(Color.green);  // Ah, nature!

        // Draw the 3 polygons
        for (i = 0; i < 3; i++) {
            g.setColor(randomGreen());
            g.drawPolygon(drawPolygons[i]);
            // Since we can't set line thickness (sheesh!),
            // let's thicken it up by drawing it again, offset 1 point
            for (j = 1; j < n; j++)
                drawPolygons[i].xPoints[j]++;
            g.setColor(randomGreen());
            g.drawPolygon(drawPolygons[i]);
        }
    }

    public void drawScaled(Graphics g, int x, int y, int width, int height) {
        drawCentered(g, x, y, width, height);
    }
}
