// TankView.java
// By Ned Etcode
// Copyright 1995, 1996 Netscape Communications Corp.  All rights reserved.

import netscape.application.*;
import netscape.util.*;

/** TankView receives dropped items and handles background and all animation
  * drawing.
  *
  * When a DragView is dropped, we retrieve the DragView's item, which in
  * this case is a subclass of DrawingSequence. We then start the animation.
  *
  * Since the TankView becomes the owner of the sequence, it therefore gets
  * notified every time one of the sequence's frame number changes, and is
  * then responsible for drawing the frame. Since there are many animations
  * going on which might overlap, we just always call the main drawing
  * routine, with the clipping rectangle set to the union of the frame's
  * previous and current position.
  */
public class TankView extends View implements DrawingSequenceOwner, DragDestination, Target {
    static final int maxSequences = 32;   // Limit the number of animations
    Bitmap           water;               // The tiled water bitmap
    Bitmap           sand;                // The tiled sand bitmap
    Bitmap           frame;
    Bitmap           side;
    Bitmap           rock;
    Bitmap           rockText;
    Rect             sandRect;            // Portion of view covered by sand
    Rect             waterRect;           // Portion of view covered by water
    Rect             leftSideRect;
    Rect             rightSideRect;
    Rect             noFrameRect;
    Rect             tankRect;
    Rect             rockRect;
    Rect	     supplyRect;
    Rect	     invoiceRect;
    DrawingSequence  sequences[];         // List of all the sequences
    int              numSequences;        // How many sequences we own
    boolean          running, inRock;
    static final int frameLeftWidth = 21,
                     frameRightWidth = 17,
                     frameTopHeight = 16,
                     frameBottomHeight = 16,
                     sandOffset = 20;

    public TankView(int x, int y, int width, int height) {
        super(x, y, width, height);

        int w, h;

        // Load get our water and sand bitmaps
        water = Bitmap.bitmapNamed("WaterTile.gif");
        sand = Bitmap.bitmapNamed("BotTile.gif");
        frame = Bitmap.bitmapNamed("Frame.gif");
        side = Bitmap.bitmapNamed("SideTile.gif");
        rock = Bitmap.bitmapNamed("Rock.gif");
        rockText = Bitmap.bitmapNamed("RockRoll.gif");

        // The sand goes on the bottom, and water everywhere else

        leftSideRect = new Rect(frameLeftWidth,
                                frameTopHeight,
                                side.width(),
                                frame.height()
				- (frameTopHeight + frameBottomHeight
				    + sand.height() - sandOffset));
        rightSideRect = new Rect(frame.width()
	                         - (frameRightWidth + side.width()),
                                 frameTopHeight,
                                 side.width(),
                                 leftSideRect.height);
        sandRect = new Rect(frameLeftWidth,
                            frame.height()
			    - (sand.height() + frameBottomHeight),
                            frame.width() - (frameLeftWidth + frameRightWidth),
                            sand.height());
        waterRect = new Rect(leftSideRect.maxX(),
                             frameTopHeight,
                             rightSideRect.x - leftSideRect.maxX(),
                             leftSideRect.height);
        noFrameRect = new Rect(leftSideRect.maxX(),
                               frameTopHeight,
                               rightSideRect.x - leftSideRect.maxX(),
                               frame.height()
			       - (frameTopHeight + frameBottomHeight));
        tankRect = new Rect(frameLeftWidth,
                            frameTopHeight,
                            frame.width() - (frameLeftWidth + frameRightWidth),
                            frame.height()
			    - (frameTopHeight + frameBottomHeight));
        rockRect = new Rect(leftSideRect.maxX() - 10,
                            sandRect.y - (sandOffset + 10),
                            rock.width(),
                            rock.height());
	supplyRect = new Rect(rockRect.x + 35, rockRect.y + 20, 85, 25);
	invoiceRect = new Rect(supplyRect.x, supplyRect.maxY(), 75, 20);
        // Allocate the array which holds all the dropped animation sequences
        sequences = new DrawingSequence[maxSequences];

        // Use buffering for smoother animation
        setBuffered(true);
    }

    public void drawView(Graphics g) {
        int  i;
        Rect bounds;
        Rect clipRect = g.clipRect();

        // First redraw the background (within the Graphics's clipping rect)
        if (clipRect.intersects(leftSideRect))
            side.drawTiled(g, leftSideRect);
        if (clipRect.intersects(rightSideRect))
            side.drawTiled(g, rightSideRect);
        if (clipRect.intersects(waterRect)) {
            g.pushState();
            g.setClipRect(waterRect);
            water.drawTiled(g, waterRect);
            g.popState();
        }

        if (clipRect.intersects(sandRect))
            sand.drawTiled(g, sandRect);

        if (!noFrameRect.contains(clipRect))
            frame.drawAt(g, 0, 0);

        if (clipRect.intersects(rockRect)) {
            rock.drawAt(g, rockRect.x, rockRect.y);
            if (inRock) {
                rockText.drawAt(g, rockRect.x, rockRect.y);
            }
        }

        // Now redraw every DrawingSequence which intersects the clipping rect
        for (i = 0; i < numSequences; i++) {
            bounds = ((Movable)sequences[i]).currentBounds();
            if (clipRect.intersects(bounds))
                sequences[i].drawAt(g, bounds.x, bounds.y);
        }
    }

    public boolean isTransparent() {
        return false;
    }

    public DragDestination acceptsDrag(DragSession session, int x, int y) {
        // If an icon is being dragged to us, return the DragDestination
        // (the TankView itself.)
        if ("icon".equals(session.dataType())) {
            return this;
        } else {
            return null;
        }
    }

    public boolean dragDropped(DragSession session) {
        IconView        item;
        DrawingSequence seq;
        Rect            dragBounds;

        // Return if we've received the maximum number of sequences
        if (numSequences == maxSequences) {
            return false;
        } else if (!tankRect.contains(session.destinationBounds())) {
            return false;
        }

        // Get a reference to the source of the drop and make a new
        // sequence from it
        item = (IconView)session.data();
        seq = item.newSequence();

        ((Aquarium)Application.application()).purchaseItem(item.info);
        // Convert the drop position into our local coordinates.  The x and y
        // fields in DragView are the mouse coordinates, but in this case
        // we want to the upper left corder of the dragView.
        dragBounds = session.destinationBounds();

        // Set the sequence's initial position in our coordinate space
        ((Movable)seq).setPosition(dragBounds.x, dragBounds.y);

        // Set the sequence's maximum x and y position
        ((Movable)seq).setTankBounds(tankRect.x, tankRect.y,
	                             tankRect.width, tankRect.height);

        sequences[numSequences++] = seq;

        // Start the animation going
        seq.setOwner(this);
        if (running) {
            seq.start();
        }

        return true;
    }

    public void performCommand(String command, Object data) {
        if ("toggleSwitch".equals(command)) {
            toggleSwitch();
        }
    }

    void toggleSwitch() {
        int i;

        running = !running;
        for (i = 0; i < numSequences; i++) {
            if (running) {
                sequences[i].start();
            } else {
                sequences[i].stop();
            }
        }
    }

    public boolean dragEntered(DragSession session) {
        return true;
    }

    public void dragExited(DragSession session) {
    }

    public boolean dragMoved(DragSession session) {
        return true;
    }

    /** This gets called every time one of the sequences' frame number
      * changes. Since they can overlap on screen, we call our main drawing
      * method, which will draw all the sequences that happen to be in the
      * old and new locations of the sequence which needs to be redrawn.
      */
    public void drawingSequenceFrameChanged(DrawingSequence aSequence) {
        Rect currentRect = ((Movable)aSequence).currentBounds();
        Rect oldRect = ((Movable)aSequence).lastBounds();

        draw(currentRect.unionRect(oldRect));
    }

    /** Don't need to do anything here, but there must be an implementation
      * since the method is part of the DrawingSequenceOwner interface
      */
    public void drawingSequenceCompleted(DrawingSequence aSequence) {
    }

    /** When the view is added into the view hierarchy, start all the
      * animations.
      */
    protected void ancestorWillRemoveFromViewHierarchy(View removedView) {
        int i;

        for (i = 0; i < numSequences; i++)
            sequences[i].stop();
        super.ancestorWillRemoveFromViewHierarchy(removedView);
    }

    /** When the view is removed from the view hierarchy, stop all the
      * animations.
      */
    protected void ancestorWasAddedToViewHierarchy(View addedView) {
        int i;

        for (i = 0; i < numSequences; i++)
            sequences[i].start();
        super.ancestorWasAddedToViewHierarchy(addedView);
    }

    public void mouseEntered(MouseEvent event) {
        if (supplyRect.contains(event.x, event.y) ||
	    invoiceRect.contains(event.x, event.y)) {
            inRock = true;
            draw(rockRect);
        }
    }

    public void mouseExited(MouseEvent event) {
        if (inRock) {
            inRock = false;
            draw(rockRect);
        }
    }

    /** Highlight the text on the rock */
    public void mouseMoved(MouseEvent event) {
        boolean newInRock = supplyRect.contains(event.x, event.y) ||
	                    invoiceRect.contains(event.x, event.y);

        if (newInRock != inRock) {
            inRock = newInRock;
            draw(rockRect);
        }
    }

    /** When the user clicks on the rock show one of the two windows */
    public boolean mouseDown(MouseEvent event) {
        if (inRock) {
            if (supplyRect.contains(event.x, event.y)) {
                ((Aquarium)Application.application()).showSupplies();
            } else if (invoiceRect.contains(event.x, event.y)) {
                ((Aquarium)Application.application()).showInvoice();
            }
        }
        return false;
    }

}
