// PasswordApp.java
// By Ned Etcode
// Copyright 1995, 1996 Netscape Communications Corp.  All rights reserved.

import netscape.application.*;
import netscape.util.*;

/* An application subclass to initialize the applet.
 * This class demonstrates how to:
 *
 * - Use the PasswordField class
 * - How to implement a TextFieldOwner class
 * - How to use popups
 */
public class PasswordApp extends Application
                         implements Target, TextFieldOwner, WindowOwner {
    PasswordField passwordField;          // The password field
    TextField     replaceTextField;       // TextField to input a string
    Button        replaceButton;          // Button to replace the
                                          // PasswordField's selected Range with
                                          // the string in replaceTextField
    Button        stringForRangeButton;   // Button to print the string in the
                                          // PasswordField's selected Range
    Popup         justificationPopup;     // Popup to set the PasswordField
                                          // justification
    Popup         drawableCharacterPopup; // Popup to set the drawable character

    /** This method gets called to initialize an application. We'll take
      * this opportunity to set up the View hierarchy.
      */
    public void init() {
        RootView rootView;

        super.init();

        rootView = mainRootView();

        // Change the color of the root view to be the standard
        // application lightGray.
        rootView.setColor(Color.lightGray);

        // Setup the password field.
        passwordField = new PasswordField(10, 10, 240, 20);
        rootView.addSubview(passwordField);

        // The application object is the owner and the target
        passwordField.setOwner(this);
        passwordField.setTarget(this);
        passwordField.setCommand("passwordFieldCommit");

        // Set a default value in the password field
        passwordField.setStringValue("Hello World");

        // Setup the popup that controls the password field's
        // justification
        justificationPopup = new Popup(10, 40, 115,20);
        justificationPopup.addItem("Left Aligned", "setLeftJustified");
        justificationPopup.addItem("Centered", "setCentered");
        justificationPopup.addItem("Right Aligned", "setRightJustified");
        justificationPopup.setTarget(this);
        rootView.addSubview(justificationPopup);

        // Setup the popup that controls the password field's
        // drawable character
        drawableCharacterPopup = new Popup(135, 40, 115, 20);
        drawableCharacterPopup.addItem("*", "setDrawableCharacter");
        drawableCharacterPopup.addItem("x", "setDrawableCharacter");
        drawableCharacterPopup.addItem(" ", "setDrawableCharacter");
        drawableCharacterPopup.setTarget(this);
        rootView.addSubview(drawableCharacterPopup);

        // Setup the TextField used as an input to replace
        // some value into the PasswordField
        replaceTextField = new TextField(10, 80, 240, 20);
        rootView.addSubview(replaceTextField);

        // Setup the button that replaces the selected text in
        // the password field with the replaceTextField's contents.
        replaceButton = new Button(135, 110, 115, 20);
        replaceButton.setTarget(this);
        replaceButton.setCommand("replace");
        replaceButton.setTitle("Replace");
        rootView.addSubview(replaceButton);

        // Setup the button that displays the password field's
        // contents into replaceTextField
        stringForRangeButton = new Button(10, 110, 115, 20);
        stringForRangeButton.setTarget(this);
        stringForRangeButton.setCommand("stringForRange");
        stringForRangeButton.setTitle("Get selection");
        rootView.addSubview(stringForRangeButton);
    }

    /** This is the sole method of the Target interface. Since java doesn't
      * have a way to do callbacks, this ugly little bit of code needs to
      * exist. The convention we recommend is to simply invoke the method
      * with the same name as the command.
      */
    public void performCommand(String command, Object data) {
        if ("textFieldCommit".equals(command))
            textFieldCommit(data);
        else if ("replace".equals(command))
            replace(data);
        else if ("stringForRange".equals(command))
            stringForRange(data);
        else if ("setLeftJustified".equals(command))
            setLeftJustified(data);
        else if ("setCentered".equals(command))
            setCentered(data);
        else if ("setRightJustified".equals(command))
            setRightJustified(data);
        else if ("setDrawableCharacter".equals(command))
            setDrawableCharacter(data);
    }

    /** Set the password justification to be left justified */
    public void setLeftJustified(Object data) {
        setPasswordJustification(Graphics.LEFT_JUSTIFIED);
    }

    /** Set the password justification to be centered */
    public void setCentered(Object data) {
        setPasswordJustification(Graphics.CENTERED);
    }

    /** Set the password justification to be right justified */
    public void setRightJustified(Object data) {
        setPasswordJustification(Graphics.RIGHT_JUSTIFIED);
    }

    /** This method is called when the password field commit
     *  to a new value
     */
    public void textFieldCommit(Object data) {
        System.out.println("PasswordField commit to new value:" +
                           passwordField.stringValue());
    }

    /** This method changes the password field's drawable character
     *  according to the selected value in the drawable character
     *  popup. */
    public void setDrawableCharacter(Object data) {
        ListItem selectedItem = drawableCharacterPopup.selectedItem();
        String   stringValue;

        if( selectedItem != null &&
            (stringValue = selectedItem.title()).length() > 0 )
            passwordField.setDrawableCharacter(stringValue.charAt(0));
    }

    /** This method replaces the selected text into the password field
     *  with the contents of replaceTextField
     */
    public void replace(Object data) {
        Range r = passwordField.selectedRange();

        if (!r.isNullRange()) {
            passwordField.replaceRangeWithString(
                r, replaceTextField.stringValue());
            // Set the selection to enclose the inserted string
            passwordField.selectRange(
                new Range(r.index, replaceTextField.stringValue().length()));
        } else
            Alert.runAlertInternally(Alert.notificationImage(),"Replace",
                                     "You must select some text into the\n" +
                                     "password field to use this button", "OK",
                                     null, null);
    }

    /** This method gets the selected text into the password field */
    public void stringForRange(Object arg) {
        Range r = passwordField.selectedRange();

        if (!r.isNullRange())
            replaceTextField.setStringValue(passwordField.stringForRange(r));
        else
            Alert.runAlertInternally(Alert.notificationImage(), "Get selection",
                                     "You must select some text into the\n" +
                                     "password field to use this button","OK",
                                     null, null);
    }

    /** This method set justification on the password field */
    public void setPasswordJustification(int justification) {
        passwordField.setJustification(justification);
        passwordField.setDirty(true);
    }

    /** Sent by the password field upon entering edit mode.
     */
    public void textEditingDidBegin(TextField textField) {
        System.out.println("TextEditing did begin");
    }

    /** Sent by the password field each time that the user changes its contents.
      */
    public void textWasModified(TextField textField) {
        System.out.println("Text was modified, value is now " + passwordField.stringValue());
    }

    /** Sent by the password field each time the text editing is about to
     *  terminate editing
     */
    public boolean textEditingWillEnd(TextField textField, int endCondition,
                                      boolean contentsChanged) {
        System.out.println("TextEditingWillEnd");
        return true;
    }

    /** Sent by the password field each tom the text editing did end */
    public void textEditingDidEnd(TextField textField, int endCondition,
                                  boolean contentsChanged) {
        System.out.println("TextEditingDidEnd value is " + textField.stringValue());
    }


    /** Sent just before a <b>aWindow</b> becomes visible. Returning
      * <b>false</b> prevents the Window from appearing onscreen.
      */
    public boolean windowWillShow(Window aWindow) {
        return true;
    }

    /** Sent just after <b>aWindow</b> has become visible.
      */
    public void windowDidShow(Window aWindow) {
    }

    /** Sent just before <b>aWindow</b> is hidden. Returning <b>false</b>
      * prevents the Window from hiding. Ask the user to confirm with an
      * Alert.
      */
    public boolean windowWillHide(Window aWindow) {
        if (Alert.runAlertInternally(Alert.warningImage(), "Quit",
                                     "Do you really want to quit?",
                                     "Yes", "No", null) ==
            Alert.DEFAULT_OPTION)
            return true;
        else
            return false;
    }

    /** Sent just after <b>aWindow</b> has hidden.
     *  Ask the application to stop running. This will
     *  cause run() to return and the application to
     *  terminate.
     */
    public void windowDidHide(Window aWindow) {
        Application.application().stopRunning();
    }

    /** Sent just after <b>aWindow</b> becomes the "main" Window, the Window
      * that receives keyboard events.
      */
    public void windowDidBecomeMain(Window aWindow) {
    }

    /** Sent just after <b>aWindow</b> ceases being the "main" Window.
      */
    public void windowDidResignMain(Window aWindow) {
    }

    /** Sent just before <b>aWindow</b> changes size by <b>deltaSize</b>.
      * The Window owner can affect this resize by modifying <b>deltaSize</b>.
      */
    public void windowWillSizeBy(Window aWindow, Size deltaSize) {
    }

    /** This method allows PasswordApp to run as a stand alone application.
      */
    public static void main(String args[]) {
        PasswordApp app;
        ExternalWindow mainWindow;
        Size size;

        app = new PasswordApp();
        mainWindow = new ExternalWindow();
        mainWindow.setTitle("Password");
        mainWindow.setOwner(app);
        app.setMainRootView(mainWindow.rootView());
        size = mainWindow.windowSizeForContentSize(260, 160);
        mainWindow.sizeTo(size.width, size.height);
        mainWindow.show();

        app.run();
        System.exit(0);
    }
}
