// CinemaView.java
// Copyright 1996 Netscape Communications Corp.  All rights reserved.

import netscape.application.*;
import netscape.util.*;

/** CinemaView is a PresentationView subclass which by default uses an
  * ImageBorder class for its border and an "outer space" DrawingSequence for
  * its background image.  It animates its text by scrolling it slowly over
  * the background.
  */
public class CinemaView extends PresentationView implements Target {
    static final String SCROLL = "TimedScroll";
    Timer               timer;            // Controls timing of animation
    SimpleTextView      textView;         // Displays the text
    int                 scrollPosition;   // Current scrollPosition
    int                 index;            // Index of current string

    // --------------  Constructors and initialization  --------------

    public CinemaView (int x, int y, int width, int height, String file) {
        /* initialize the PresentationView to use a "curtain" image for
         * the border, and an animated "outer space" image for the background
         */
        super(x, y, width, height,
              new ImageBorder(Bitmap.bitmapNamed("curtain.gif"), 42, 16),
              new SpaceImage());

        Rect interiorRect;

        /* initialize the text view */
        interiorRect = interiorRect();
        textView = new SimpleTextView(interiorRect.x + 20,
                                     interiorRect.y + interiorRect.height,
                                     interiorRect.width - 40,
                                     interiorRect.height);
        textView.setFont(new Font("TimesRoman", Font.PLAIN, 24));
        textView.setTextColor(Color.cyan);
        addSubview(textView);

        /* initialize the timer to send us a performCommand message every 140
         * milliseconds
         */
        timer = new Timer(this, SCROLL, 140);
        isRunning = false;
        setBlankLinesMarkParagraphs(true); // Join groups of lines together
        if (file != null) {
            loadFile(file);               // Try to load file if specified
        }
    }

    public CinemaView(String file) {
        this(0, 0, 200, 200, file);
    }


    // --------------  Animation control methods  --------------

    /** Set the text to the first string and start the scrolling animation. */
    public void start() {
        index = 0;

        /* position the text at the bottom of the interior area */
        scrollPosition = height() - border().bottomMargin();
        textView.moveTo(textView.bounds.x, scrollPosition);
        textView.setStringValue((String)(strings.elementAt(index)));
        super.start();
        timer.start();
    }

    /** Stop the text scrolling. */
    public void stop() {
        super.stop();
        timer.stop();
        textView.setStringValue(null);
    }

    /** Pause/unpause the text scrolling. */
    public void pause() {
        if (isRunning) {
            timer.stop();
        } else {
            timer.start();
        }
        super.pause();
    }

    /** Adjust the speed of the text scrolling. */
    public void setSpeed(float speedFactor) {
        timer.setDelay((int)(160.0 / speedFactor));
    }

    public void performCommand(String command, Object anObject) {
        if (command.equals(SCROLL)) {
            scrollPosition--;
            if (-scrollPosition + border().topMargin() > textView.height()) {
                /* move the scroll position back to the bottom and go to
                 * the next string.
                 */
                scrollPosition = height() - border().bottomMargin();
                if (++index == strings.count()) {
                    index = 0;
                }
                textView.setStringValue((String)(strings.elementAt(index)));
            }

            /* repostion the textView to the new position */
            textView.moveTo(textView.bounds.x, scrollPosition);

            /* note that we are not redrawing the text.  This is an
             * optimization made possible because we are using
             * an animated background which is updating at a much
             * higher rate.  When the background is redrawn, the
             * textView will get automatically updated as well.
             */
        }
    }
}
