// ImageBorder.java
// Copyright 1996 Netscape Communications Corp.  All rights reserved.


import netscape.application.*;
import netscape.util.*;

/** A Border subclass which displays a tiled Image.  Notes whether the
  * Image is a DrawingSequence so that the Border's user can animate it.
  */
public class ImageBorder extends Border  {
    Image   image;           // The image being used to tile the border
    int     borderWidth;     // The width of the left and right borders
    int     borderHeight;    // The height of the top and bottom borders
    boolean imageIsAnimated; // True if the image is a DrawingSequence
    Rect    drawRect;        // Used to store the current border rectangle
    int     frameRate;       // The initial frame rate of the image

    // --------------  Constructors and initialization  --------------

    public ImageBorder(Image image, int width, int height) {
        borderWidth = width;
        borderHeight = height;
        drawRect= new Rect();
        setImage(image);
    }

    public ImageBorder(Image image) {
        this(null, (image != null) ? image.width() : 0,
             (image != null) ? image.height() : 0);
    }

    public void setImage(Image anImage) {
        this.image = anImage;
        if (image != null) {
            if (image instanceof DrawingSequence) {
                imageIsAnimated = true;
                frameRate = ((DrawingSequence)image).frameRate();
            }
            if (borderWidth <= 0) {
                borderWidth = image.width();
            }
            if (borderHeight <= 0) {
                borderHeight = image.height();
            }
        }
    }

    public Image image() {
        return image;
    }

    public boolean isAnimated() {
        return imageIsAnimated;
    }


    // --------------  Animation control methods  --------------

    /** This will be set by the PresentationView which uses this Border. */
    public void setImageOwner(DrawingSequenceOwner owner) {
        if (imageIsAnimated) {
            ((DrawingSequence)image).setOwner(owner);
        }
    }

    /** If the Image is a DrawingSequence, start the animation. */
    public void start() {
        if (imageIsAnimated) {
            ((DrawingSequence)image()).start();
        }
    }

    /** If the Image is a DrawingSequence, stop the animation. */
    public void stop() {
        if (imageIsAnimated) {
            ((DrawingSequence)image()).stop();
        }
    }

    // If the image is a DrawingSequence, pause/unpause the animation
    public void pause() {
        if (((DrawingSequence)image()).isAnimating()) {
            ((DrawingSequence)image()).stop();
        } else {
            ((DrawingSequence)image()).start();
        }
    }

    /** If the Image is a DrawingSequence, adjust the frame rate. */
    public void setSpeed(float speedFactor) {
        if (imageIsAnimated) {
            float rate = (float)frameRate / speedFactor;
            ((DrawingSequence)image()).setFrameRate((int)rate);
        }
    }


    //  ---------------------  Border methods  ---------------------

    /** Primitive method that returns width of the left side of the border. */
    public int leftMargin() {
        return borderWidth;
    }

    /** Primitive method that returns width of the right side of the border. */
    public int rightMargin() {
        return borderWidth;
    }

    /** Primitive method that returns height of the top side of the border. */
    public int topMargin() {
        return borderHeight;
    }

    /** Primitive method that returns height of the bottom side. */
    public int bottomMargin() {
        return borderHeight;
    }

    public void drawInRect(Graphics g, int x, int y, int width, int height) {
        Rect clipRect = g.clipRect();

        if (image == null) {
            return;
        }

        /* draw the left border if needed */
        drawRect.setBounds(x, y, borderWidth, height);
        if (drawRect.intersects(clipRect)) {
            g.pushState();
            g.setClipRect(drawRect, true);
            image.drawTiled(g, drawRect);
            g.popState();
        }

        /* draw the right border if needed */
        drawRect.setBounds(x + width - borderWidth, y, borderWidth, height);
        if (drawRect.intersects(clipRect)) {
            g.pushState();
            g.setClipRect(drawRect, true);
            image.drawTiled(g, drawRect);
            g.popState();
        }

        /* draw the top border if needed */
        drawRect.setBounds(x + borderWidth, y, width - 2 * borderWidth,
                           borderHeight);
        if (drawRect.intersects(clipRect)) {
            g.pushState();
            g.setClipRect(drawRect, true);
            image.drawTiled(g, drawRect);
            g.popState();
        }

        /* draw the bottom border if needed */
        drawRect.setBounds(x + borderWidth, y + height - borderHeight,
                           width - 2 * borderWidth, borderHeight);
        if (drawRect.intersects(clipRect)) {
            g.pushState();
            g.setClipRect(drawRect, true);
            image.drawTiled(g, drawRect);
            g.popState();
        }
    }
}
