// PortalView.java
// Copyright 1996 Netscape Communications Corp.  All rights reserved.


import netscape.application.*;
import netscape.util.*;

/** PortalView is a PresentationView subclass which by default uses an
  * ImageBorder class for its border and an "blue sky" DrawingSequence for
  * its background image.  It animates its text by using two SimpleTextView
  * instances to simulate cross-fading.
  */
public class PortalView extends PresentationView implements Target {
    SimpleTextView textView1;       // Used to display the text
    SimpleTextView textView2;
    Timer          timer;           // Controls cross-fading of the text
    long           referenceTime;   // Records timestamp of a crossfade start
    long           currentTime;     // Current timestamp minus referenceTime
    long           fadeTime;        // Duration of a single crossfade
    long           pauseTime;       // Records time of a pause
    int            index;           // Index of current string
    Color          backgroundColor; // The color of the background
    int            r, g, b;         // The rgb values of the background color
    float          interp;          // Text color interpolation factor
    Rect           textBounds;      // The union of both text bounds

    // --------------  Constructors and initialization  --------------

    public PortalView(int x, int y, int width, int height, String file) {
        /* initialize the PresentationView to have an ImageBorder and
         * a SkyImage DrawingSequence for its background image
         */
        super(x, y, width, height,
             new ImageBorder(Bitmap.bitmapNamed("wood.gif"), 16, 24),
             new SkyImage());

        Rect interiorRect;

        /* initilize the text views to the size of the interior, with a bit
         * of left and right margin space
         */
        interiorRect = interiorRect();
        textView1 = new SimpleTextView(interiorRect.x + 5, interiorRect.y,
                                      interiorRect.width - 10,
                                      interiorRect.height);
        textView2 = new SimpleTextView(interiorRect.x + 5, interiorRect.y,
                                      interiorRect.width - 10,
                                      interiorRect.height);

        backgroundColor = ((SkyImage)image()).backgroundColor();

        r = backgroundColor.red();
        g = backgroundColor.green();
        b = backgroundColor.blue();

        textView1.setFont(new Font("TimesRoman", Font.BOLD, 36));
        textView1.setTextColor(backgroundColor);
        textView2.setFont(new Font("TimesRoman", Font.BOLD, 36));
        textView2.setTextColor(Color.black);
        addSubview(textView1);
        addSubview(textView2);

        /* initialize the timer to run every 250 milliseconds */
        timer = new Timer(this, null, 200);
        fadeTime = 10000;                  // The time for one crossfade
        index = 0;
        currentTime = 0;
        referenceTime = -1;
        if (file != null) {
            loadFile(file);               // Load the file is specified
        }
    }

    public PortalView(String file) {
        this(0, 0, 200, 200, file);
    }


    // --------------  Animation control methods  --------------

    /** Set the first text view to the first string, and start the animation.
      */
    public void start() {
        referenceTime = -1;
        index = 0;
        currentTime = 0;
        pauseTime = -1;
        interp = 0.0f;
        timer.start();
        textView1.setTextColor(backgroundColor);
        textView2.setTextColor(Color.black);
        if (strings != null) {
            textView1.setStringValue((String)strings.elementAt(index));
            textView2.setStringValue(null);
            centerText();
        }
        super.start();
    }

    /** Stop the animation. */
    public void stop() {
        super.stop();
        timer.stop();
        textView1.setStringValue(null);
        textView2.setStringValue(null);
        draw();
    }

    /** Pause/unpause the animation. */
    public void pause() {
        if (isRunning) {
            timer.stop();
            pauseTime = timer.timeStamp() - referenceTime;
        } else {
            timer.start();
        }
        super.pause();
    }

    /** Adjust the speed of the animation. Rather than adjust the time speed,
      * We simply adjust the cross-fade time, leaving the timed events at the
      * same rate.
      */
    public void setSpeed(float speedFactor) {
        fadeTime = (int)(8000.0 / speedFactor);
    }

    /** Adjust the crossfade.  Since we only have one level of transparency
      * to work with, we have to simulate crossfading.  Otherwise, the
      * text that's fading out could occlude the text that's fading up.
      * We use two overlapping text fields, and simply switch their
      * order in the view hierarchy at the moment when they have the same
      * text color, so that the darker one is always at the bottom of the
      * view hierarchy and therefore gets drawn last.
      */
    public void performCommand(String command, Object object) {
        Color color;
        long  lastTime = currentTime;
        float lastInterp = interp;
        float i;

        if (referenceTime <= 0)
            /* we're just starting, so note the current timestamp */
            referenceTime = timer.timeStamp();

        if (pauseTime > 0) {
            /* we're recovering from a pause, so adjust the referenceTime */
            referenceTime = timer.timeStamp() - pauseTime;
            pauseTime = -1;
        }

        /* current time within one crossfade */
        currentTime = timer.timeStamp() - referenceTime;

        /* "interp" oscillates between 0.0 and 1.0 */
        if (currentTime < fadeTime) {
            interp = (float)currentTime / (float)fadeTime;
        } else if (currentTime < (2 * fadeTime)) {
            interp = (float)((2 * fadeTime) - currentTime) / (float)fadeTime;
            if (lastTime < fadeTime) {
                if (++index == strings.count()) {
                    index = 0;
                }
                if (strings != null) {
                   textView2.setStringValue((String)strings.elementAt(index));
                   centerText();
                }
            }
        } else {
            /* we've completed a complete fade-up/fade-down cycle, so
             * move to the next line of text
             */
            referenceTime = timer.timeStamp();
            interp = 0.0f;
            if (++index == strings.count()) {
                index = 0;
            }
            if (strings != null) {
                textView1.setStringValue((String)strings.elementAt(index));
                centerText();
            }
        }

        if ((lastInterp <= 0.5) && (interp > 0.5)) {
            /* textView1 is about to have the darker text color, so put
             * it on the bottom of the view hierarchy so it gets drawn last
             */
            disableDrawing();
            textView1.removeFromSuperview();
            addSubview(textView1);
            reenableDrawing();
        } else if ((lastInterp >= 0.5) && (interp < 0.5)) {
            /* textView2 is about to have the darker text color, so put
             * it on the bottom of the view hierarchy so it gets drawn last
             */
            disableDrawing();
            textView2.removeFromSuperview();
            addSubview(textView2);
            reenableDrawing();
        }

        color = new Color((int)(interp * r), (int)(interp * g),
                          (int)(interp * b));
        textView2.setTextColor(color);

        interp = 1.0f - interp;
        color = new Color((int)(interp * r), (int)(interp * g),
                          (int)(interp * b));
        textView1.setTextColor(color);

        draw(textBounds);
    }

    /** Recenter both text fields in the interior area. */
    public void centerText() {
        Rect interiorRect = interiorRect();
        textView1.setBounds(interiorRect.x + 5,
                             interiorRect.y +
                             (interiorRect.height -
                              textView1.bounds.height) / 2,
                             interiorRect.width - 10,
                             textView1.bounds.height);
        textView2.setBounds(interiorRect.x + 5,
                             interiorRect.y +
                             (interiorRect.height -
                              textView2.bounds.height) / 2,
                             interiorRect.width - 10,
                             textView2.bounds.height);
        textBounds = textView1.bounds.unionRect(textView2.bounds);
    }

    /** If we resize, we need to recenter the text views. */
    public void didSizeBy(int deltaWidth, int deltaHeight) {
        super.didSizeBy(deltaWidth, deltaHeight);
        centerText();
    }
}
