// PresentationView.java
// Copyright 1996 Netscape Communications Corp.  All rights reserved.


import netscape.application.*;
import netscape.util.*;
import java.net.*;
import java.io.*;

/** PresentationView, as a subclass of ContainerView, can accept a Border
  * and a background Image. PresentationView also implements the
  * DrawingSequenceOwner interface in order to handle the drawing of
  * animated border images and background images.
  *
  * It also accepts DragViews dropped onto it which contain file
  * names.  The files are read in, parsed into individual lines, and
  * stored in a vector for the subclass to display.
  */
public class PresentationView extends ContainerView
                      implements DrawingSequenceOwner, DragDestination {
    boolean borderIsAnimated;          // Whether the border image is animated
    boolean imageIsAnimated;           // Whether background image is animated
    int     imageFrameRate;            // Initial rate of background image
    Rect    borderRect;                // Stores various border rects
    boolean isRunning;                 // Animation is running
    String  fileName;                  // File name for loading strings
    Vector  strings;                   // The strings to display
    boolean ignoreEmptyLines;          // If true, ignore blank lines in file
    boolean blankLinesMarkParagraphs;  // If true, join lines separated by
                                       //      blank lines into single lines

    // --------------  Constructors and initialization  --------------

    public PresentationView(int x, int y, int width, int height,
                            Border border, Image image) {
        super(x, y, width, height);

        setBuffered(true);        // Avoid flicker
        setHorizResizeInstruction(WIDTH_CAN_CHANGE); // Resize with the window
        setVertResizeInstruction(HEIGHT_CAN_CHANGE);
        setBorder(border);                // Set our border
        setImage(image);                  // Set our background image
        isRunning = false;
        strings = new Vector();
        ignoreEmptyLines = true;          // Skip blank lines in text file
        blankLinesMarkParagraphs = false; // Display one line at a time
    }

    public PresentationView(int x, int y, int width, int height) {
        this(x, y, width, height, null, null);
    }

    public PresentationView() {
        this(0, 0, 50, 50, null, null);
    }

    /** Set the border.  If the Border is an ImageBorder, check to see if
      * its image is animated (i.e., a DrawingSequence of some kind).
      */
    public void setBorder(Border border) {
        super.setBorder(border);
        if (border != null) {
            borderRect = new Rect(0, 0, 0, 0);
            if (border instanceof ImageBorder) {
                ((ImageBorder)border).setImageOwner(this);
                borderIsAnimated = ((ImageBorder)border).isAnimated();
            } else {
                borderIsAnimated = false;
            }
        } else {
            borderIsAnimated = false;
        }
    }

    /** Set our background image.  Also check to see if it is actually
      * some kind of DrawingSequence.  If so, set ourself as the owner
      * and set the image to be scaled to cover the background area.
      */
    public void setImage(Image image) {
        super.setImage(image);
        imageIsAnimated = false;

        if (image == null) {
            return;
        }

        if (image instanceof DrawingSequence) {
            imageIsAnimated = true;
            ((DrawingSequence)image).setOwner(this); // We handle drawing
            setImageDisplayStyle(Image.SCALED);  // Cover the whole background
            imageFrameRate = ((DrawingSequence)image).frameRate();
        } else {
            setImageDisplayStyle(Image.TILED);
        }
    }

    /** If true, skip blank lines in the text file.
      */
    public void setIgnoreEmptyLines(boolean flag) {
        ignoreEmptyLines = flag;
    }

    /** If true, concatenate lines separated by blank lines into a single line.
      */
    public void setBlankLinesMarkParagraphs(boolean flag) {
        blankLinesMarkParagraphs = flag;
    }


    // --------------  Animation control methods  --------------

    /** Start animating.  Start the border and image, necessary.
      * Subclasses should override this method to start their own animation.
      */
    public void start() {
        if (borderIsAnimated) {
            ((ImageBorder)border()).start();
        }
        if (imageIsAnimated) {
            ((DrawingSequence)image()).start();
        }
        isRunning = true;
    }

    /** Stop animating.  Stop the border and image, necessary.
      * Subclasses should override this method to stop their own animation.
      */
    public void stop() {
        if (borderIsAnimated) {
            ((ImageBorder)border()).stop();
        }
        if (imageIsAnimated) {
            ((DrawingSequence)image()).stop();
        }
        isRunning = false;
    }

    /** Pause animating.  Pause the border and image, necessary.
      * Subclasses should override this method to pause their own animation.
      */
    public void pause() {
        if (isRunning) {
            if (borderIsAnimated) {
                ((ImageBorder)border()).stop();
            }
            if (imageIsAnimated) {
                ((DrawingSequence)image()).stop();
            }
        } else {
            if (borderIsAnimated) {
                ((ImageBorder)border()).start();
            }
            if (imageIsAnimated) {
                ((DrawingSequence)image()).start();
            }
        }
        isRunning = !isRunning;
    }

    /** Subclasses should override this method to set their animation speed. */
    public void setSpeed(float speedFactor) {
        float   rate;

        if (borderIsAnimated) {
            ((ImageBorder)border()).setSpeed(speedFactor);
        }
        if (imageIsAnimated) {
            rate = (float)imageFrameRate / speedFactor;
            ((DrawingSequence)image()).setFrameRate((int)rate);
        }
    }

    /** Stop the animation before being yanked from the view hierarchy. */
    public void removeFromSuperview() {
        stop();
        super.removeFromSuperview();
    }


    // --------------  DrawingSequenceOwner methods  ------------------

    /** Handle the drawing of animated borders and background images. */
    public void drawingSequenceFrameChanged(DrawingSequence aSequence) {
        if (borderIsAnimated &&
            (aSequence == ((ImageBorder)border()).image())) {
            int w = border().leftMargin();
            int h = border().topMargin();

            /* draw the rectangles for the four borders */
            borderRect.setBounds(0, 0, width(), h);
            draw(borderRect);
            borderRect.setBounds(0, height() - h, width(), height());
            draw(borderRect);
            borderRect.setBounds(0, h, w, height() - h - h);
            draw(borderRect);
            borderRect.setBounds(width() - w, h, w, height() - h - h);
            draw(borderRect);
        }
        else if (aSequence == image()) {
            /* draw the current frame of the background image */
            if (border() != null) {
                draw(interiorRect());
            } else {
                draw();
            }
        }
    }

    /** Unused. Must be present because it's part of DrawingSequenceOwner. */
    public void drawingSequenceCompleted(DrawingSequence aSequence) {
    }


    // --------------  Drag and drop support methods  ------------------

    public DragDestination acceptsDrag(DragSession session, int x, int y) {
        return this;
    }

    public boolean dragEntered(DragSession session) {
        return (session.data() != null) && (session.data() instanceof String);
    }

    public boolean dragMoved(DragSession session) {
        return (session.data() != null) && (session.data() instanceof String);
    }

    public void dragExited(DragSession session) {
    }

    /** Accept dropped DragViews which contain the names of files to load. */
    public boolean dragDropped(DragSession session) {
        boolean wasRunning = isRunning;
        Object data = session.data();

        if ((data == null) || !(data instanceof String)) {
            return false;
        }

        if (isRunning) {
            stop();
        }

        loadFile((String)data);

        if (wasRunning) {
            start();
        }

        return true;
    }

    /** Load the specified file into the strings array. */
    public void loadFile(String file) {
        URL             url = null;
        DataInputStream stream = null;
        String          line, s;

        fileName = file;

        if (rootView() == null)
            /* can't get to the application object, so we'll wait 'til later.*/
            return;

        try {
            /* create a URL which points to the file, which is presumed to
             * live in the same directory as the application code files
             */
            url = new URL(Application.application().codeBase(), file);
        }
        catch (MalformedURLException ex) {
            url = null;
        }

        /* if we're joining multiple lines together, create a temporary
         * string to hold the lines.
         */
        s = blankLinesMarkParagraphs ? new String() : null;

        if (url != null) try {
            /* open a stream to the URL */
            stream = new DataInputStream(url.openStream());
            if (stream != null) {
                /* remove any existing strings */
                strings.removeAllElements();

                while (true) {
                    /* read the next line */
                    line = stream.readLine();
                    /* if null, we're at the end of the file */
                    if (line == null) {
                        break;
                    }

                    /* get rid of leading and trailing spaces */
                    line = line.trim();

                    if (line.length() > 0) {
                        if (blankLinesMarkParagraphs) {
                            /* add the line to the temporary string */
                            if (s == null) {
                                s = new String();
                            }
                            s += (" " + line);
                        } else {
                            /* add the line directly to the strings vector */
                            addString(line);
                        }
                    } else {
                        if (blankLinesMarkParagraphs) {
                            /* add the temporary string to the strings vector*/
                            addString(s);
                            s = null;  // clear the temporary string
                        }
                        if (!blankLinesMarkParagraphs && !ignoreEmptyLines) {
                            /* add the blank line as well, in this case */
                            addString(line);
                        }
                    }
                }
                /* make sure we get the last line */
                if (blankLinesMarkParagraphs && (s != null)) {
                    addString(s);
                }
                stream.close();
            }
        }
        catch (IOException ex) {
        }
    }

    /** We may have not been able to load the file at initialization
      * because we weren't yet part of the view hierarchy, and therefore
      * couldn't get to the application via the rootView, so load it now.
      */
    protected void ancestorWasAddedToViewHierarchy(View addedView) {
        if ((fileName != null) && (strings.count() == 0)) {
            loadFile(fileName);
        }
    }


    // ----------------------  Other methods  ----------------------

    /** Add a string to the display string vector. */
    public void addString(String string) {
        strings.addElement(string);
    }

    /** The strings for the subclass to display. */
    public Vector strings() {
        return strings();
    }

    /** Returns the interior rectangle, the area not covered by the borders. */
    public Rect interiorRect() {
        return (border() == null) ?
            new Rect(0, 0, bounds.width, bounds.height) :
            border().interiorRect(0, 0, bounds.width, bounds.height);
    }
}
