// PresentationWindow.java
// Copyright 1996 Netscape Communications Corp.  All rights reserved.


import netscape.application.*;
import netscape.util.*;

/** PresentationWindow is a subclass of InternalWindow which displays a
  * PresentationView, a set of Buttons and a Slider.  The buttons control
  * the animation of the PresentationView.
  */
public class PresentationWindow extends InternalWindow {
    static final String STOP = "Stop";
    static final String START = "Start";
    static final String PAUSE = "Pause";
    static final String SPEED = "Speed";
    PresentationView    presentationView; // The PresentationView subclass
    Button              playButton;       // Starts animation when pressed
    Button              stopButton;       // Stops animation when pressed
    Button              pauseButton;      // Pauses animation when pressed
    Slider              slider;           // Adjusts speed of animation
    Image               playImage1;       // Images used for Play button
    Image               playImage2;
    ImageSequence       pauseSequence;    // DrawingSequence used for Pause

    // --------------  Constructors and initialization  --------------

    public PresentationWindow(int x, int y, int width, int height,
                              String title, PresentationView view) {
        /* initialize window to have title bar and bottom border */
        super(x, y, width, height);
        int buttonY;

        if (title != null) {
            setTitle(title);            // Set the window's title
        }
        setResizable(true);             // Allow resizing
        setPresentationView(view);      // Add the PresentationView

        /* compute the y position of the buttons */
        buttonY = contentView().bounds.height - 20;

        /* add the Stop button */
        stopButton = new Button(0, buttonY, 55, 20);
        stopButton.setTitle("Stop");
        stopButton.setTarget(this);
        stopButton.setCommand(STOP);
        stopButton.setVertResizeInstruction(TOP_MARGIN_CAN_CHANGE);
        stopButton.setEnabled(false);
        stopButton.setImage(Bitmap.bitmapNamed("stop.gif"));
        stopButton.setImagePosition(Button.IMAGE_ON_LEFT);
        addSubview(stopButton);

        /* add the Play button */
        playButton = new Button(55, buttonY, 55, 20);
        playButton.setTitle("Start");
        playButton.setTarget(this);
        playButton.setCommand(START);
        playButton.setVertResizeInstruction(TOP_MARGIN_CAN_CHANGE);
        playImage1 = Bitmap.bitmapNamed("play1.gif");
        playImage2 = Bitmap.bitmapNamed("play2.gif");
        playButton.setImage(playImage1);
        playButton.setImagePosition(Button.IMAGE_ON_LEFT);
        addSubview(playButton);

        /* add the Pause button */
        pauseButton = new Button(110, buttonY, 55, 20);
        pauseButton.setTitle("Pause");
        pauseButton.setType(Button.TOGGLE_TYPE);
        pauseButton.setTarget(this);
        pauseButton.setCommand(PAUSE);
        pauseButton.setVertResizeInstruction(TOP_MARGIN_CAN_CHANGE);
        pauseButton.setEnabled(false);

        /* the button's image will be an ImageSequence */
        pauseSequence = new ImageSequence(pauseButton);
        pauseSequence.addImage(Bitmap.bitmapNamed("pause1.gif"));
        pauseSequence.addImage(Bitmap.bitmapNamed("pause2.gif"));
        pauseSequence.setPlaybackMode(DrawingSequence.FORWARD_LOOP);
        pauseSequence.setFrameCount(2);
        pauseSequence.setFrameRate(500);
        pauseButton.setImage(pauseSequence);
        pauseButton.setImagePosition(Button.IMAGE_ON_LEFT);
        addSubview(pauseButton);

        /* add the speed slider */
        slider = new Slider(170, buttonY, width - 175, 20);
        slider.setTarget(this);
        slider.setCommand(SPEED);
        slider.setLimits(25, 200); // Values expressed in percent (25% to 200%)
        slider.setValue(100);     // Initialize to 100%
        slider.setVertResizeInstruction(TOP_MARGIN_CAN_CHANGE);
        slider.setHorizResizeInstruction(WIDTH_CAN_CHANGE);
        addSubview(slider);
    }

    public PresentationWindow(int x, int y, int width, int height) {
        this(x, y, width, height, null, null);
    }

    public void setPresentationView(PresentationView view) {
        if (view != null) {
            if (presentationView != null) {
                /* only allow one per window */
                presentationView.removeFromSuperview();
            }
            presentationView = view;
            presentationView.setBounds(0, 0, contentView().bounds.width,
                                       contentView().bounds.height - 20);
            addSubview(presentationView);
        }
    }


    // -----------  Target interface method  -------------------

    /** Process commands sent by the Buttons when they are pressed. */
    public void performCommand(String command, Object object) {
        if (command.equals(START)) {
            playButton.setImage(playImage2); // Set to the "lit" version
            if (pauseButton.state()) {
                pauseButton.setState(false); // Turn off the pause button
            }
            stopButton.setEnabled(true);     // Enable the stop button
            pauseButton.setEnabled(true);    // Enable the pause button
            presentationView.start();        // Start the animations
        } else if (command.equals(STOP)) {
            playButton.setImage(playImage1); // Set to the "unlit" version
            playButton.draw();               // Redraw to show new image
            if (pauseButton.state()) {
                pauseButton.setState(false); // Turn off the pause button
            }
            stopButton.setEnabled(false);    // Disable the stop button
            pauseButton.setEnabled(false);   // Disable the pause button
            pauseSequence.stop();            // Stop pause button flashing
            pauseSequence.setCurrentFrameNumber(0);
            pauseButton.draw();              // Redraw to show correct image
            presentationView.stop();         // Stop the animations
        } else if (command.equals(PAUSE)) {
            presentationView.pause();     // Pause/unpause the animations
            if (pauseButton.state()) {
                pauseSequence.start();    // Start the pause button flashing
            } else {
                pauseSequence.stop();     // Stop the pause button flashing
                pauseSequence.setCurrentFrameNumber(0);
            }
        } else if (command.equals(SPEED)) {
            /* adjust the speed of the animations, converting the
             * slider's percentage numbers to a 0.0 - 1.0 value
             */
            presentationView.setSpeed((float)slider.value() * 0.01f);
        }
    }
}
