// Presenter.java
// Copyright 1996 Netscape Communications Corp.  All rights reserved.

import netscape.application.*;
import netscape.util.*;

/** This example illustrates how the Internet Foundation Classes may be
  * used to easily create animated presentations.  The Timer class is used
  * to automate scrolling, cross-fading, and other animating of text.  The
  * ContainerView class is used together with the Border class to
  * implement a variety of frame styles.
  *
  * We subclass InternalWindow to create PresentationWindow
  * PresentationWindow, which contains our subclass of ContainerView,
  * called PresentationView. PresentationView as well as some buttons
  * and a slider used to control the animation.  PresentationView.java
  * PresentationView implements the DrawingSequenceOwner interface in
  * order to handle the drawing of animated border images and background
  * images.  It also accepts DragViews dropped onto it which contain file
  * names.  The files are read in, parsed into individual lines, and
  * stored in a vector for the subclass to display.
  *
  * There are three subclasses of PresentationView which animate their
  * text in various ways, CinemaView.java CinemaView, MarqueeView.java
  * MarqueeView, and PortalView.java PortalView.  Each has been preloaded
  * with one of the files, but you may change the text by dragging a
  * different file from the file icons on the background "desktop".
  *
  * Some platforms may not be able to run more than one or two of the
  * presentations at a time without slowing down.  Also, the graphics look
  * best using at least 16-bit (65k) color settings.
  *
  * This example primarily illustrates the use of the following classes
  * and concepts:
  *
  * Timers
  * Borders
  * ContainerViews
  * Buttons and Sliders
  * Drag and Drop sessions
  *
  * Secondarily, it also illustrates:
  *
  * View Hierarchy
  * DrawingSequence and DrawingSequenceOwner
  */
public class Presenter extends Application {

    public void init() {
        PresentationWindow window;

        super.init();
        /* set the background to be our special "tunnel" Image subclass */
        mainRootView().setImage(new TunnelImage());
        mainRootView().setImageDisplayStyle(Image.SCALED);
        mainRootView().setBuffered(true);

        /* add a file icon to the background for each text file */
        mainRootView().addSubview(new FileIcon(10, 10, "yow"));
        mainRootView().addSubview(new FileIcon(70, 10, "concepts"));
        mainRootView().addSubview(new FileIcon(130, 10, "important"));

        /* set up the "Marquee" window, initialize the MarqueeView with the
         * file "yow"
         */
        window = new PresentationWindow(10, 85, 390, 405, "Marquee",
                                        new MarqueeView("yow"));
        window.show();

        /* set up the "Portal" window, initialize the PortalView with the
         * file "important"
         */
        window = new PresentationWindow(410, 45, 275, 445, "Portal",
                                        new PortalView("important"));
        window.show();

        /* set up the "Cinema" window, initialize the CinemaView with the
         * file "concepts"
         */
        window = new PresentationWindow(120, 500, 479, 270, "Cinema",
                                        new CinemaView("concepts"));
        window.show();
    }

    /** These two methods allow Presenter to run as a stand alone application.
      */
    public static void main(String args[]) {
        Presenter app;
        Size size;
        ExternalWindow mainWindow;

        app = new Presenter();
        mainWindow = new ExternalWindow();
        app.setMainRootView(mainWindow.rootView());
        size = mainWindow.windowSizeForContentSize(750, 650);
        mainWindow.sizeTo(size.width, size.height);
        mainWindow.show();

        app.run();
                System.exit(0);
    }
}


