// ScreenView.java
// Copyright 1996 Netscape Communications Corp.  All rights reserved.

import netscape.application.*;
import netscape.util.*;

/** ScreenView is a view which uses a Timer to animate the drawing of a
  * grid of lines.  It cycles through three modes: Wipe mode gradually
  * reveals the grid under a black rectangle, opening to the right;
  * Dazzle mode cycles through 3 slightly offset grids to create a flickering
  * effect; Blink mode alternates a grid with a blank rectangle.
  * The idea is to impose a sort of billboard light-array effect over
  * whatever view is underneath.
  */
public class ScreenView extends View implements Target {
    int        mode;             // The current animation mode
    Timer      timer;            // The timer controlling the animation
    long       referenceTime;    // Start timestamp of an animation sequence
    long       currentTime;      // Current timestamp within a sequence
    long       segmentTime;      // Length of a mode sequence
    int        blinkDelay;       // Blink time during blink mode
    long       pauseTime;        // Records time when a pause happens
    Polygon    polygons[];       // The grid polygons
    Rect       wipeRect;         // The wipe mode painting rectangle
    boolean    blinkState;       // blink on/off state
    int        polygonCounter;   // polygon counter for dazzle mode
    boolean    isRunning;        // True if timer is running

    /** Animation modes. */
    static int          WIPE_MODE = 0, DAZZLE_MODE = 1, BLINK_MODE = 2;


    // --------------  Constructors and initialization  --------------

    /** Constructs a ScreenView. */
    public ScreenView(int x, int y, int width, int height) {
        super(x, y, width, height);
        setPolygons();         // Initialize the grid polygons
        currentTime = -1;
        segmentTime = 4000;    // Normal time for each mode is 4 seconds
        blinkDelay = 500;      // Blink on and off once per second
        timer = new Timer(this, "", 200); // Send performCommand every 200 ms
        isRunning = false;
    }


    // --------------  Animation control methods  --------------

    /** Start the animation. */
    public void start() {
        // Start in wipe mode, completely black
        mode = WIPE_MODE;
        wipeRect.setBounds(1, 1, bounds.width - 1, bounds.height - 1);
        wipeRect.setBounds(0, 0, bounds.width, bounds.height);
        timer.setDelay(200);
        currentTime = -1;
        pauseTime = -1;
        timer.start();
        isRunning = true;
    }

    /** Stop the animation. */
    public void stop() {
        timer.stop();
        isRunning = false;
    }

    /** Pause/resume the animation. */
    public void pause() {
        if (isRunning) {
            timer.stop();
            pauseTime = timer.timeStamp() - referenceTime;
        } else {
            timer.start();
        }
        isRunning = !isRunning;
    }

    /** Set the animation rate.  Instead of adjusting the timer rate,
      * we just adjust the segment time and blink delay (for BLINK mode).
      */
    public void setSpeed(float speedFactor) {
        segmentTime = (int)(4000.0 / speedFactor);
        blinkDelay = (int)(500.0 / speedFactor);
    }


    /** Sent by the Timer at each delay increment. */
    public void performCommand(String command, Object object) {
        if (currentTime < 0)
            /* note the timestamp if we've just started */
            referenceTime = timer.timeStamp();

        if (pauseTime > 0) {
            /* adjust the reference time after a pause */
            referenceTime = timer.timeStamp() - pauseTime;
            pauseTime = -1;
        }

        /* current time within a mode */
        currentTime = timer.timeStamp() - referenceTime;

        if (currentTime > segmentTime) {
            /* time to change modes */
            if (mode == WIPE_MODE) {
                mode = DAZZLE_MODE;
                polygonCounter = 0;
                timer.setDelay(100);
                wipeRect.x = 1;
                wipeRect.width = bounds.width - 1;
            } else if (mode == DAZZLE_MODE) {
                mode = BLINK_MODE;
                blinkState = false;
                timer.setDelay(blinkDelay);
            } else if (mode == BLINK_MODE) {
                mode = WIPE_MODE;
                timer.setDelay(200);
                /* tell the superview that we've gotten through all 3 modes */
                ((MarqueeView)superview()).advance();
            }
            referenceTime = timer.timeStamp();
            currentTime = currentTime - segmentTime;
        }
        draw();
    }

    //  -------------------  View drawing methods  -------------------

    /** Draw the ScreenView. */
    public void drawView(Graphics g) {
        g.setColor(Color.black);

        if (mode == WIPE_MODE) {
            g.drawPolygon(polygons[0]);   // Just use the first polygon
            g.fillRect(wipeRect);         // Black out part of it
            wipeRect.x = (int)(((float)currentTime / (float)segmentTime)
                               * (float)width());
            wipeRect.width = bounds.width - wipeRect.x - 1;
        } else if (mode == DAZZLE_MODE) {
            /* cycle through the polygons */
            g.drawPolygon(polygons[polygonCounter++]);
            if (polygonCounter == polygons.length) {
                polygonCounter = 0;
            }
        } else if (mode == BLINK_MODE) {
            if (blinkState) {
                g.drawPolygon(polygons[0]);
            } else {
                g.fillRect(wipeRect);
            }
            blinkState = !blinkState;
        }
    }


    //  -----------------------  Other methods  --------------------

    /** Create three slightly-offset polygons.  Each polygon is a grid
      * of doubled lines, 4 pixels apart.
      */
    public void setPolygons() {
        Polygon polygon;
        int i, j, x, y, w, h;

        w = width() - 1;
        h = height() - 1;

        if (polygons == null) {
            polygons = new Polygon[4];
            wipeRect = new Rect(1, 1, w, h);
        } else {
            wipeRect.setBounds(1, 1, w, h);
        }

        for (i = 0; i < 4; i++) {
            polygon = new Polygon();
            for (x = 2 + i; x < w; x += 4) {
                polygon.addPoint(x, -2);
                polygon.addPoint(x, h + 2);
                polygon.addPoint(x + 1, h + 2);
                polygon.addPoint(x + 1, -2);
            }
            for (y = - i; y < h; y += 4) {
                polygon.addPoint(-2, y);
                polygon.addPoint(w + 2, y);
                polygon.addPoint(w + 2, y);
                polygon.addPoint(-2, y);
            }

            polygons[i] = polygon;
        }
    }

    /** If we change size, we have to make new grids. */
    public void didSizeBy(int deltaWidth, int deltaHeight) {
        super.didSizeBy(deltaWidth, deltaHeight);
        setPolygons();
    }
}
