// SimpleTextView.java
// Copyright 1996 Netscape Communications Corp.  All rights reserved.

import netscape.application.*;
import netscape.util.*;
import java.util.StringTokenizer;

/** A simple text-drawing View.  Pre-wraps the given text by parsing it into a
  * Vector of Strings, one per line.  Pre-determines the x offset of each line.
  * When the text value is reset, it automatically rewraps the new text
  * and resets the view's height to fit the new text.
  */
public class SimpleTextView extends View {
    String      value;          // The current string value
    Vector      lines;          // The string value parsed into lines
    int         lineX[];        // The x-offset of each line
    Color       bgColor;        // The background color, if any
    Color       color;          // The text color
    Font        font;           // The text font
    FontMetrics metrics;        // The font metrics of the above font
    int         lineHeight;     // The total height of a line in the font
    int         justification;  // The justification type


    // --------------  Constructors and initialization  --------------

    public SimpleTextView(int x, int y, int width, int height) {
        super(x, y, width, height);
        bgColor = Color.white;
        color = Color.black;
        setFont(new Font("Helvetica", Font.PLAIN, 14));
        justification = Graphics.CENTERED;
        lines = new Vector();
        setHorizResizeInstruction(WIDTH_CAN_CHANGE);
    }

    // If our font changes, we need to reparse the lines, and reset our
    // height according to the number of lines and the line height
    public void setFont(Font aFont) {
        font = aFont;
        metrics = font.fontMetrics();
        lineHeight = metrics.maxAscent() + metrics.maxDescent() +
                                                        metrics.leading();
        if (value != null) {
            parseLinesForWidth(width());
            sizeTo(width(), lines.count() * lineHeight);
        }
    }

    public void setTextColor(Color aColor) {
        color = aColor;
    }

    public void setBackgroundColor(Color aColor) {
        bgColor = aColor;
    }

    public void setJustification(int value) {
        justification = value;
    }

    public void setStringValue(String aString) {
        value = aString;

        if (value != null) {
            parseLinesForWidth(width());
            sizeTo(width(), lines.count() * lineHeight);
        } else {
            lines.removeAllElements();
            sizeTo(width(), 0);
            return;
        }
    }

    public String stringValue() {
        return value;
    }

    /** Parse the string value into a Vector of Strings, one per line according
      * to the current View width.  Also compute the x offset of each line
      * according to the justification style.
      */
    public void parseLinesForWidth(int width) {
        StringTokenizer tokenizer;   // Breaks the string into words
        String          token;       // The current word
        int             tokenWidth;  // The width of the current word
        int             i, x;        // temproary variables
        int             lineWidth;   // The width of the current line
        int             numLines;    // The total number of lines
        StringBuffer    buffer;      // Used to assemble the lines

        if (value == null) {
            return;
        }

        tokenizer = new StringTokenizer(value, " \t\n\r", true);
        buffer = new StringBuffer();

        lines.removeAllElements();
        x = 0;

        /* assemble the words into a vector of lines so that each line is
         * less wide than the view's width
         */
        while (tokenizer.hasMoreTokens()) {
            token = tokenizer.nextToken();
            tokenWidth = metrics.stringWidth(token);
            if ((x + tokenWidth) < width) {
                x += tokenWidth;
            } else {
                lines.addElement(buffer.toString());
                buffer.setLength(0);
                x = tokenWidth;
            }
            buffer.append(token);
        }

        /* make sure we get the last line */
        if (buffer.length() > 0)
            lines.addElement(buffer.toString());

        numLines = lines.count();

        /* now compute the x offset of each line according to the
         * justification style
         */
        if ((lineX == null) || (numLines > lineX.length))
            lineX = new int[numLines];

        if (justification == Graphics.CENTERED) {
            for (i = 0; i < numLines; i++) {
                lineWidth = metrics.stringWidth((String)(lines.elementAt(i)));
                lineX[i] = (width - lineWidth) / 2;
            }
        } else {
            for (i= 0; i < numLines; i++) {
                lineX[i] = 0;
            }
        }
    }

    /** Draw all the lines of text.
      */
    public void drawView(Graphics g) {
        int     i, n, y, width, minY, maxY;

        width = width();

        if (!isTransparent() && (bgColor != null)) {
            g.setColor(bgColor);
            g.fillRect(g.clipRect());
        }

        g.setColor(color);
        g.setFont(font);
        n = lines.count();

        minY = g.clipRect().y;
        maxY = minY + g.clipRect().height + lineHeight;
        y = metrics.ascent();
        for (i = 0; i < n; i++) {
            if (!((y < minY) || (y > maxY))) {
                g.drawString((String)(lines.elementAt(i)), lineX[i], y);
            }
            y += lineHeight;
        }
    }

    /** If our width changes, we need to reparse the lines, and reset our
      * height according to the number of lines and the line height.
      */
    public void setBounds(int x, int y, int width, int height) {
        if (width != bounds.width) {
            parseLinesForWidth(width);
        }
        if (lines != null) {
            height = lines.count() * lineHeight;
        }
        super.setBounds(x, y, width, height);
    }
}
